/* -*- linux-c -*- (Autoset linux indent style when using emacs) */

/*
 * Copyright 1999 Digi International (www.digi.com)
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *	This is the support for reading/writing the audio tap.
 *
 *	Typical use:
 *		cat /dev/dg/dgdm/audiotap0 > /dev/audio &
 *		echo "ulaw 0 0x4000 0x4000" > /dev/dg/dgdm/audiotap0
 *			   ^
 *			   (dsp txgain rxgain)
 *
 *	Control of the device is done by writing to it.
 *	Please see include/audiotap.h for details on what is
 *	currently accepted by the FEP when writing to the device.
 *
 *	$Id: dgdm_audiotap.c,v 1.3 2000/04/04 19:48:32 randall Exp $
 */

#define __NO_VERSION__
#include "dgdm_driver.h"
#include "audiotap.h"


/* external variables */
extern int		dgdm_debug;


/* Prototypes for THIS file */
STATIC int		audio_openack_cb(DXB *dxb, int channel, int error_code,
                                         DXB_ARG arg);
STATIC int		audio_closeack_cb(DXB *dxb, int channel, DXB_ARG arg);
STATIC void		*audiotap_copyout(void *dst, uint8 *src, int len,
                                          DXB_ARG arg);
STATIC int		audio_data_cb(DXB *dxb, int channel, int data_len,
                                      DXB_ARG arg);





/**************************************************************************
 *
 * Functions
 *
 **************************************************************************/

/*
 * dgdm_audiotap_open()
 *
 * Open the device
 */
GLOBAL int dgdm_audiotap_open(BOARD_T *brd)
{
        if(!brd)
                return(-ENODEV);

	brd->audio.eof = 0;

	/*
	 * No error checking here... if its not the first
	 * open we are more than happy to return with sucess.
	 * In reality, we should only allow 1 process to have
	 * this device open for reading, but multiple writers
	 * are AOK.
	 */
	if(brd->audio.opencnt++)
		return(0);

	/*
	 * First open, increment module use count
	 */
	MOD_INC_USE_COUNT;

	/*
	 * Loop until we can get a DXB open message queued.
	 * We use the "rxbuf" option of libdxb to manage a
	 * received data circular buffer for us.
	 */
	for (;;) {
		dxb_lock(brd);

		brd->audio.dxbchan = dxb_rxbuf_open(brd->dxb,
						    AUDIOTAP_NAME,
						    0 /*fixed cost*/, NULL, 0,
						    NULL, AUDIOBUF_SIZE);

		if(brd->audio.dxbchan >= 0)
			break;

		dxb_unlock(brd);

		if(dgdm_sleep(1)) {
			return(-EINTR);
		}
	}

	/*
	 * Return credits and setup callbacks
	 */
	dxb_return_credits(brd->dxb, brd->audio.dxbchan, AUDIOBUF_SIZE - 1);

	dxb_set_openack_cb(brd->dxb, brd->audio.dxbchan,
		audio_openack_cb, (DXB_ARG) brd);

	dxb_set_data_cb(brd->dxb, brd->audio.dxbchan,
		audio_data_cb, (DXB_ARG) brd);

	dxb_unlock(brd);

	/*
	 * Wait for open acknowledgement to come back,
	 * then check its error code.
	 */
	interruptible_sleep_on(&brd->audio.openack_wait);
	if(current_got_fatal_signal())
		return(-EINTR);

	if(brd->audio.error_code) {
		DPR_BASIC(("dgdm_audiotap_open() DXB open failed %d\n",
			brd->audio.error_code));
		return(-ENODEV);
	} else {
		DPR_BASIC(("dgdm_audiotap_open() DXB open was acked\n"));
	}

	return(0);
}



/*
 * audio_openack_cb()
 *
 * callback we get on the open acknowledgement
 */
STATIC int audio_openack_cb(DXB *dxb, int channel, int error_code, DXB_ARG arg)
{
        BOARD_T *brd = (BOARD_T *) arg;

        if(!brd)
                return(-ENODEV);

        brd->audio.error_code = error_code;
        wake_up_interruptible(&brd->audio.openack_wait);
        return(0);
}



/*
 * dgdm_audiotap_close()
 *
 * Close the device
 */
GLOBAL void dgdm_audiotap_close(BOARD_T *brd)
{
	int	i;
	int	rc;

        if(!brd)
                return;

	/*
	 * If its not the last close, just return.
	 */
        if(--brd->audio.opencnt > 0)
                return;

	DPR_BASIC(("dgdm_audiotap_close() last close called\n"));

        /*
         * If the DXB open never completed, we are done
         */
        if(brd->audio.dxbchan < 0) {
                MOD_DEC_USE_COUNT;
                return;
        }

        /*
         * Attempt to queue close message up to 100 times
         * If we can't get it thru by then, just give up
         * because we don't want to hang the user.
         */
        dxb_set_closeack_cb(brd->dxb, brd->audio.dxbchan,
			    audio_closeack_cb, (DXB_ARG) brd);

        for (i = 0; i < 100; ++i) {
                dxb_lock(brd);
                rc = dxb_close(brd->dxb, brd->audio.dxbchan);
                dxb_unlock(brd);

                /*
                 * If close message was queued, wait for ACK and
                 * then blow out of here.
                 */
                if(rc == 0) {
                        interruptible_sleep_on(&brd->audio.closeack_wait);
                        break;
                }

                /*
                 * Wait 1 tick and try again.  If user mashes
                 * interrupt key, blow out of here
                 */
                if(dgdm_sleep(1))
                        break;
        }

	MOD_DEC_USE_COUNT;
}



/*
 * audio_closeack_cb()
 *
 * Callback we get on a close ack DXB message.
 */
STATIC int audio_closeack_cb(DXB *dxb, int channel, DXB_ARG arg)
{
        BOARD_T *brd = (BOARD_T *) arg;

        if(!brd)
                return(-ENODEV);

        wake_up_interruptible(&brd->audio.closeack_wait);
        return(0);
}



/*
 * dgdm_audiotap_read()
 *
 * Read from the device
 */
GLOBAL int dgdm_audiotap_read(BOARD_T *brd, READ_PROTO)
{
	int	avail;

        if(!brd)
                return(-ENODEV);

	if(count == 0)
		return(0);

	/*
	 * Wait for desired amount of data to become available
	 */
	for (;;) {
		avail = dxb_rxbuf_count(brd->dxb, brd->audio.dxbchan);
		if(avail)
			break;
		if(brd->audio.eof)
			return(0);
		interruptible_sleep_on(&brd->audio.read_wait);
		if(current_got_fatal_signal())
			return(-EINTR);
	}

	count = MIN(count, avail);

	/*
	 * Set copyout function and move the data from the
	 * "rxbuf" to user space.  Then return the DXB credits.
	 */
	dxb_rxbuf_set_copyout(brd->dxb, brd->audio.dxbchan,
			      audiotap_copyout, (DXB_ARG) 0);

	count = dxb_rxbuf_read(brd->dxb, brd->audio.dxbchan, buf, count);
	dxb_return_credits(brd->dxb, brd->audio.dxbchan, count);
	ADD_POS(count);
	return(count);
}



/*
 * dgdm_audiotap_write()
 *
 * Write to the device.  This is a control function.
 */
GLOBAL int dgdm_audiotap_write(BOARD_T *brd, WRITE_PROTO)
{
	char	data[64];

	DPR_WRITE(("audiotap_write: len=%d\n", count));

        if(!brd)
                return(-ENODEV);

	if(count > sizeof(data))
		count = sizeof(data);

	COPY_FROM_USER(data, buf, count);

	for (;;) {
		dxb_lock(brd);
		if(dxb_space_avail(brd->dxb, brd->audio.dxbchan, 
				    0, count, 0) == DXB_OK)
			break;
		dxb_unlock(brd);
		dgdm_sleep(1);	/* FIXME: wait for credit return! */
	}

	dxb_data(brd->dxb, brd->audio.dxbchan, count);
	dxb_write_data_block(brd->dxb, data, count);
	dxb_write_done(brd->dxb);

	dxb_unlock(brd);

	DPR_WRITE(("audiotap_write: wrote %d\n", count));

	ADD_POS(count);
	return(count);
}



/*
 * audiotap_copyout()
 *
 * Custom "copyout" function we will use for dxb_rxbuf_read().
 *
 * This one copies the data to user space.
 */
STATIC void *audiotap_copyout(void *dst, uint8 *src, int len, DXB_ARG arg)
{
	char	*cdst = (char *) dst;

	COPY_TO_USER(cdst, src, len);
	return(cdst + len);
}



/*
 * audio_data_cb()
 *
 * Callback function for data.
 *
 * Copy the data to the rxbuf, then wakeup the reader.
 */
STATIC int audio_data_cb(DXB *dxb, int channel, int data_len, DXB_ARG arg)
{
	BOARD_T	*brd = (BOARD_T *) arg;
	int	rc;

        if(!brd)
                return(-ENODEV);

        if(data_len)
                rc = dxb_rxbuf_copyin(dxb, channel, data_len);
	else
		brd->audio.eof = 1;	/* 0 length message == EOF */

	dxb_read_done(dxb);
        wake_up_interruptible(&brd->audio.read_wait);
	return(0);
}
