/* -*- linux-c -*- (Autoset linux indent style when using emacs) */

/*
 * Copyright 1999 by Digi International (www.digi.com)
 *    Jeff Randall <Jeff_Randall@digi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/*
 *	This is the support for reading/writing the eye quality monitor.
 *
 *	Typical use:
 *		cat /dev/dg/dgdm/eqm0 > file &
 *		echo "eqm ascii 0 100 0" > /dev/dg/dgdm/eqm0
 *			   ^
 *			   (format dsp rate nsamples)
 *
 *	Control of the device is done by writing to it.
 *	Please see include/eqm.h for details on what is
 *	currently accepted by the FEP when writing to the device.
 */

 /* $Id: dgdm_eqm.c,v 1.3 2000/04/11 22:34:07 randall Exp $ */


#define __NO_VERSION__
#include "dgdm_driver.h"
#include "eqm.h"


/* external variables */
extern int		dgdm_debug;


/* prototypes for THIS file */
GLOBAL int		dgdm_eqm_open(BOARD_T *brd);
STATIC int		eqm_openack_cb(DXB *dxb, int channel, int error_code,
                                       DXB_ARG arg);
GLOBAL void		dgdm_eqm_close(BOARD_T *brd);
STATIC int		eqm_closeack_cb(DXB *dxb, int channel, DXB_ARG arg);
GLOBAL int		dgdm_eqm_read(BOARD_T *brd, READ_PROTO);
GLOBAL int		dgdm_eqm_write(BOARD_T *brd, WRITE_PROTO);
STATIC void		*eqm_copyout(void *dst, uint8 *src, int len,
                                     DXB_ARG arg);
STATIC int		eqm_data_cb(DXB *dxb, int channel, int data_len,
                                    DXB_ARG arg);





/**************************************************************************
 *
 * Functions
 *
 **************************************************************************/

/*
 * dgdm_eqm_open()
 *
 * Open the device
 */
GLOBAL int dgdm_eqm_open(BOARD_T *brd)
{

        if(!brd)
                return(-ENODEV);

	brd->eqm.eof = 0;

	/*
	 * No error checking here... if its not the first
	 * open we are more than happy to return with sucess.
	 * In reality, we should only allow 1 process to have
	 * this device open for reading, but multiple writers
	 * are AOK.
	 */
	if (brd->eqm.opencnt++)
		return (0);

	/*
	 * First open, increment module use count
	 */
	MOD_INC_USE_COUNT;

	/*
	 * Loop until we can get a DXB open message queued.
	 * We use the "rxbuf" option of libdxb to manage a
	 * received data circular buffer for us.
	 */
	for (;;) {
		dxb_lock(brd);

		brd->eqm.dxbchan = dxb_rxbuf_open(brd->dxb,
                                                  EQM_NAME, 0 /*fixed cost*/,
                                                  NULL, 0,
                                                  NULL, EQMBUF_SIZE);

		if (brd->eqm.dxbchan >= 0)
			break;

		dxb_unlock(brd);

		if (dgdm_sleep(1)) {
			return (-EINTR);
		}
	}

	/*
	 * Return credits and setup callbacks
	 */
	dxb_return_credits(brd->dxb, brd->eqm.dxbchan, AUDIOBUF_SIZE - 1);

	dxb_set_openack_cb(brd->dxb, brd->eqm.dxbchan,
                           eqm_openack_cb, (DXB_ARG) brd);

	dxb_set_data_cb(brd->dxb, brd->eqm.dxbchan,
                        eqm_data_cb, (DXB_ARG) brd);

	dxb_unlock(brd);

	/*
	 * Wait for open acknowledgement to come back,
	 * then check its error code.
	 */
	interruptible_sleep_on(&brd->eqm.openack_wait);
	if (current_got_fatal_signal())
		return (-EINTR);

	if (brd->eqm.error_code) {
		DPR_BASIC(("dgdm_eqm_open() DXB open failed %d\n",
                           brd->eqm.error_code));
		return (-ENODEV);
	} else {
		DPR_BASIC(("dgdm_eqm_open() DXB open was acked\n"));
	}

	return (0);
}



/*
 * eqm_openack_cb()
 *
 * callback we get on the open acknowledgement
 */
STATIC int eqm_openack_cb(DXB *dxb, int channel, int error_code, DXB_ARG arg)
{
        BOARD_T *brd = (BOARD_T *) arg;

        if(!brd)
                return(-ENODEV);

        brd->eqm.error_code = error_code;
        wake_up_interruptible(&brd->eqm.openack_wait);
        return (0);
}



/*
 * dgdm_eqm_close()
 *
 * Close the device
 */
GLOBAL void dgdm_eqm_close(BOARD_T *brd)
{
	int	i;
	int	rc;

        if(!brd)
                return;

	/*
	 * If its not the last close, just return.
	 */
        if (--brd->eqm.opencnt > 0)
                return;

	DPR_BASIC(("dgdm_eqm_close() last close called\n"));

        /*
         * If the DXB open never completed, we are done
         */
        if (brd->eqm.dxbchan < 0) {
                MOD_DEC_USE_COUNT;
                return;
        }

        /*
         * Attempt to queue close message up to 100 times
         * If we can't get it thru by then, just give up
         * because we don't want to hang the user.
         */
        dxb_set_closeack_cb(brd->dxb, brd->eqm.dxbchan,
                            eqm_closeack_cb, (DXB_ARG) brd);

        for (i = 0; i < 100; ++i) {
                dxb_lock(brd);
                rc = dxb_close(brd->dxb, brd->eqm.dxbchan);
                dxb_unlock(brd);

                /*
                 * If close message was queued, wait for ACK and
                 * then blow out of here.
                 */
                if (rc == 0) {
                        interruptible_sleep_on(&brd->eqm.closeack_wait);
                        break;
                }

                /*
                 * Wait 1 tick and try again.  If user mashes
                 * interrupt key, blow out of here
                 */
                if (dgdm_sleep(1))
                        break;
        }

	MOD_DEC_USE_COUNT;
}



/*
 * eqm_closeack_cb()
 *
 * Callback we get on a close ack DXB message.
 */
STATIC int eqm_closeack_cb(DXB *dxb, int channel, DXB_ARG arg)
{
        BOARD_T *brd = (BOARD_T *) arg;

        if(!brd)
                return(-ENODEV);

        wake_up_interruptible(&brd->eqm.closeack_wait);

        return (0);
}



/*
 * dgdm_eqm_read()
 *
 * Read from the device
 */
GLOBAL int dgdm_eqm_read(BOARD_T *brd, READ_PROTO)
{
	int	avail;

        if(!brd)
                return(-ENODEV);

	if (count == 0)
		return (0);

	/*
	 * Wait for desired amount of data to become available
	 * and/or half the RXBUF is full.
	 */
	for (;;) {
		avail = dxb_rxbuf_count(brd->dxb, brd->eqm.dxbchan);
		if (avail >= count || avail >= (EQMBUF_SIZE/2) )
			break;
		if (brd->eqm.eof)
			return (0);
		interruptible_sleep_on(&brd->eqm.read_wait);
		if (current_got_fatal_signal())
			return (-EINTR);
	}

	count = MIN(count, avail);

	/*
	 * Set copyout function and move the data from the
	 * "rxbuf" to user space.  Then return the DXB credits.
	 */
	dxb_rxbuf_set_copyout(brd->dxb, brd->eqm.dxbchan,
                              eqm_copyout, (DXB_ARG) 0);

	count = dxb_rxbuf_read(brd->dxb, brd->eqm.dxbchan, buf, count);

	dxb_return_credits(brd->dxb, brd->eqm.dxbchan, count);

	ADD_POS(count);

	return (count);
}



/*
 * dgdm_eqm_write()
 *
 * Write to the device.  This is a control function.
 */
GLOBAL int dgdm_eqm_write(BOARD_T *brd, WRITE_PROTO)
{
	char	data[64];

	DPR_WRITE(("dgdm_eqm_write: len=%d\n", count));

        if(!brd)
                return(-ENODEV);

	if (count > sizeof(data))
		count = sizeof(data);

	COPY_FROM_USER(data, buf, count);

	for (;;) {
		dxb_lock(brd);
		if (dxb_space_avail(brd->dxb, brd->eqm.dxbchan,
                                    0, count, 0) == DXB_OK)
			break;
		dxb_unlock(brd);
		dgdm_sleep(1);	/* FIXME: wait for credit return! */
	}

	dxb_data(brd->dxb, brd->eqm.dxbchan, count);
	dxb_write_data_block(brd->dxb, data, count);
	dxb_write_done(brd->dxb);

	dxb_unlock(brd);

	DPR_WRITE(("dgdm_eqm_write: wrote %d\n", count));

	ADD_POS(count);

	return (count);
}



/*
 * eqm_copyout()
 *
 * Custom "copyout" function we will use for dxb_rxbuf_read().
 *
 * This one copies the data to user space.
 */
STATIC void *eqm_copyout(void *dst, uint8 *src, int len, DXB_ARG arg)
{
	char	*cdst = (char *) dst;

	COPY_TO_USER(cdst, src, len);

	return (cdst + len);
}



/*
 * eqm_data_cb()
 *
 * Callback function for data.
 *
 * Copy the data to the rxbuf, then wakeup the reader.
 */
STATIC int eqm_data_cb(DXB *dxb, int channel, int data_len, DXB_ARG arg)
{
	BOARD_T	*brd = (BOARD_T *) arg;
	int	rc;

        if(!brd)
                return(-ENODEV);

        if (data_len)
                rc = dxb_rxbuf_copyin(dxb, channel, data_len);
	else
		brd->eqm.eof = 1;	/* 0 length message == EOF */

	dxb_read_done(dxb);

        wake_up_interruptible(&brd->eqm.read_wait);

	return (0);
}
