/* -*- linux-c -*- (Autoset linux indent style when using emacs) */

/*
 * Copyright Digi International (www.digi.com)
 *    Jeff Randall <Jeff_Randall@digi.com>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 * implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 * PURPOSE.  See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* $Id: dgdm_fileio.c,v 1.4 1999/12/11 00:20:38 randall Exp $ */

#define __NO_VERSION__
#include "dgdm_driver.h"


/************************************************************************
 *
 * Routines to allow kernel access to the filesystem, for
 * reading firmware files.
 *
 * NOTE: This requires that we 'hijack' the insmod's user context and is
 *       the main reason we are not offered as a static driver.
 *
 ************************************************************************/

/*
 * dgdm_file_open()
 *
 * You provide a struct file to fill in.  Other arguments are
 * the usual ones for open.
 */
GLOBAL int dgdm_file_open(struct file *f, char *filename, int flags, int mode)
{
	int		flag;
	int		error;
	struct inode	*inode;

#if LINUX_VERSION_CODE >= 0x20100
        struct dentry	*dentry;
#endif
	
	if (flags)
		return(-EINVAL); /* Only read access supported at this time */

	/*
	 * do what get_empty_filp() would do to init a struct file
	 */
	memset(f, 0, sizeof(*f));
        f->f_count = 1;

	/*
	 * Now do mostly what do_open() would do
	 */
        f->f_flags = flag = flags;
	f->f_mode = (flag+1) & O_ACCMODE;	/* Its a bit mask now */

#if LINUX_VERSION_CODE >= 0x20100
        dentry = open_namei(filename, flag, mode);
        error = PTR_ERR(dentry);
        if (IS_ERR(dentry))
                return(error);

        inode = dentry->d_inode;
        f->f_dentry = dentry;
#else
        error = open_namei(filename, flag, mode, &inode, NULL);
        if (error)
                return error;

        f->f_inode = inode;
#endif

        f->f_pos = 0;
        f->f_reada = 0;
        f->f_op = NULL;
        if (inode->i_op)
                f->f_op = inode->i_op->default_file_ops;
        if (f->f_op && f->f_op->open) {
                error = f->f_op->open(inode, f);
                if (error)
                        goto cleanup_inode_or_dentry;
        }
	f->f_flags &= ~(O_CREAT | O_EXCL | O_NOCTTY | O_TRUNC);

        if (!f->f_op || !f->f_op->read)
                goto cleanup_inode_or_dentry;

	return (0);

 cleanup_inode_or_dentry:
#if LINUX_VERSION_CODE >= 0x20100
        dput(dentry);
#else
        iput(inode);
#endif

        return(error);
}



/*
 * dgdm_file_read()
 *
 * Returns count of bytes read.
 * Shamelessly hacked from sys_read()
 */
GLOBAL int dgdm_file_read(struct file *filep, char *buf, int count)
{
	int		error;

	/*
	 * I'm not entirely sure if what I'm about to do is architecture
	 * dependant.  Basically, we are going to do a set_fs() to
	 * fool memcpy_tofs() (2.0) or copy_to_user() (2.1) to copy
	 * into kernel space.  Both of those routines will eventually
	 * be called by (*filep->f_op->read)().
	 */
#if LINUX_VERSION_CODE >= 0x20100
	{
		mm_segment_t	old_fs;

		old_fs = get_fs();
		set_fs(KERNEL_DS);

		error = filep->f_op->read(filep, buf, count, &filep->f_pos);

		set_fs(old_fs);
	}
#else
	{
		unsigned long	old_fs;

		old_fs = get_fs();
		set_fs(KERNEL_DS);

		error = filep->f_op->read(filep->f_inode, filep, buf, count);

		set_fs(old_fs);
	}
#endif

	return (error);
}



/*
 * dgdm_file_close()
 *
 */
GLOBAL void dgdm_file_close(struct file *filep)
{
#if LINUX_VERSION_CODE >= 0x20100
        dput(filep->f_dentry);
#else
        iput(filep->f_inode);
#endif
}
