
/*
 Copyright 1999-2000 by Digi International (www.digi.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*
 *	User level DXB library API
 *
 *	The goal here is to make a generic DXB library that can be
 *	used by both device drivers and the FEP.
 *
 *	To port this, you will need to create a local dxbtypes.h
 *	header file that defines some basic scalar types, and you
 *	will also need to supply a handful of functions/macros that
 *	are operating system specific.  Look at the bottom of this
 *	header file for the functions/macros you will need to supply.
 *
 *	$Id: dxblib.h,v 1.36 2000/01/10 21:24:58 emillan Exp $
 */

/*
 *	This is the maximum number of channels supported
 *
 *	NOTE: this currently *MUST* be a multiple of 32 and <= 256.
 *
 *	FEPs should probably always leave this set to the maximum.
 *	Drivers may override this value to the number of channels
 *	they will potentially open.
 */
#ifndef	DXB_MAXCHAN
	#define	DXB_MAXCHAN	256
#endif
#if DXB_MAXCHAN & 31
	#error "DXB_MAXCHAN must be a multiple of 32."
#endif
#if DXB_MAXCHAN > 256
	#error "DXB_MAXCHAN must be <= 256."
#endif

/*
 *	We want to have inline support, if possible.  These macros
 *	should handle it for us.  If you can do inlines with your
 *	compiler, then "#define DXB_HAS_INLINE 1" in dxbtypes.h.
 *	Otherwise, leave it undefined or "0"
 */
#if DXB_HAS_INLINE == 1
	#define	DXB_INLINE(PROTO, BODY) static inline PROTO BODY
#else
	#if DXB_LIBSOURCE == 1
		#define	DXB_INLINE(PROTO, BODY) PROTO BODY
	#else
		#define	DXB_INLINE(PROTO, BODY) extern PROTO ;
	#endif
#endif


/*
 *	NOTE: the DXB_MP flag is only used in conjunction with firmware
 *	builds. It is NOT the beginnings of multi-processor host driver
 *	support.
 */

/*
 *	This is the definition of a DXB queue
 *
 */
typedef struct
{
        uint32	in;		/* Producers 'in' offset */

	uint32	pad1[7];

        uint32	out;		/* Consumers 'out' offset */

	uint32	pad2[7];

        DXBBUF	buf;		/* The page(s) of the actual buffer */

	/*
	 *	Internally used by API
	 */
	uint32	in_tmp;
#ifndef	DXB_MP
	uint32	out_tmp;

	uint32	highpri;
	uint32	channel;
	int	data_len;
	int	data_cnt;
	int	ctrl_len;
	int	ctrl_cnt;
#else
	uint32	unused1[7];	/* Pad to cache line boundary */
	DXB_MPQSTATE	mpqstate[DXB_MP];
#endif

} DXBQ;

/*
 *	The DXB structure needs an early typedef because the
 *	callback routines take a DXB as an argument.
 */
typedef struct dxb DXB;

/*
 *	The user supplied argument to a callback
 */
typedef void	*DXB_ARG;

/*
 *	The callback routine when an OPEN message is received
 */
typedef int	(*DXB_OPEN_CB)(
			DXB		*dxb,
			int		channel,
			char		*devid,
			int		data_len,
			DXB_ARG		arg
		);

/*
 *	The callback routine when an OPEN acknowledgement message is received
 */
typedef int	(*DXB_OPENACK_CB)(
			DXB	*dxb,
			int	channel,
			int	error_code,
			DXB_ARG	arg
		);

/*
 *	The callback routine when a CLOSE message is received
 */
typedef int	(*DXB_CLOSE_CB)(
			DXB	*dxb,
			int	channel,
			DXB_ARG	arg
		);

/*
 *	The callback routine when an CLOSE acknowledgement message is received
 */
typedef int	(*DXB_CLOSEACK_CB)(
			DXB	*dxb,
			int	channel,
			DXB_ARG	arg
		);

/*
 *	The callback routine when a DATA message is received
 */
typedef int	(*DXB_DATA_CB)(
			DXB	*dxb,
			int	channel,
			int	data_len,
			DXB_ARG	arg
		);

/*
 *	The callback routine when a CDATA message is received
 */
typedef int	(*DXB_CDATA_CB)(
			DXB	*dxb,
			int	channel,
			int	priority,
			int	ctrl_len,
			int	data_len,
			DXB_ARG	arg
		);

/*
 *	The callback routine when a CREDIT award message is received
 */
typedef int	(*DXB_CREDIT_CB)(
			DXB	*dxb,
			int	channel,
			int	credits,	/* Total credits available */
			DXB_ARG	arg
		);

/*
 *	The callback routine when a PUSH message is received
 */
typedef int	(*DXB_PUSH_CB)(
			DXB		*dxb,
			int		channel,
			char		*module_id,
			int		data_len,
			DXB_ARG		arg
		);

/*
 *	The callback routine when a PUSH acknowledgement message is received
 */
typedef int	(*DXB_PUSHACK_CB)(
			DXB	*dxb,
			int	channel,
			int	error_code,
			DXB_ARG	arg
		);

/*
 *	The callback routine when a POP message is received
 */
typedef int	(*DXB_POP_CB)(
			DXB		*dxb,
			int		channel,
			DXB_ARG		arg
		);

/*
 *	The callback routine when a POP acknowledgement message is received
 */
typedef int	(*DXB_POPACK_CB)(
			DXB	*dxb,
			int	channel,
			int	error_code,
			DXB_ARG	arg
		);

/*
 *	The callback routine when a LINK message is received
 */
typedef int	(*DXB_LINK_CB)(
			DXB		*dxb,
			int		channel,
			int		other_channel,
			DXB_ARG		arg
		);

/*
 *	The callback routine when a LINK acknowledgement message is received
 */
typedef int	(*DXB_LINKACK_CB)(
			DXB	*dxb,
			int	channel,
			int	mux_id,
			int	error_code,
			DXB_ARG	arg
		);

/*
 *	The callback routine when a UNLINK message is received
 */
typedef int	(*DXB_UNLINK_CB)(
			DXB		*dxb,
			int		channel,
			int		mux_id,
			DXB_ARG		arg
		);

/*
 *	The callback routine when a UNLINK acknowledgement message is received
 */
typedef int	(*DXB_UNLINKACK_CB)(
			DXB	*dxb,
			int	channel,
			int	mux_id,
			int	error_code,
			DXB_ARG	arg
		);

/*
 *	The callback routine when a FLUSH message is received
 */
typedef int	(*DXB_FLUSH_CB)(
			DXB	*dxb,
			int	channel,
			int	flush_read,
			int	flush_write,
			DXB_ARG	arg
		);

/*
 *	The callback routine when a FLUSH acknowledgement message is received
 */
typedef int	(*DXB_FLUSHACK_CB)(
			DXB	*dxb,
			int	channel,
			int	error_code,
			DXB_ARG	arg
		);

/*
 *	The callback routine when a IOCTL message is received
 */
typedef int	(*DXB_IOCTL_CB)(
			DXB	*dxb,
			int	channel,
			int	command,
			int	data_len,
			DXB_ARG	arg
		);

/*
 *	The callback routine when a IOCTL acknowledgement message is received
 */
typedef int	(*DXB_IOCTLACK_CB)(
			DXB	*dxb,
			int	channel,
			int	command,
			int	data_len,
			int	error_code,
			DXB_ARG	arg
		);

/*
 *	The callback routine when a CANCEL message is received
 */
typedef int	(*DXB_CANCEL_CB)(
			DXB	*dxb,
			int	channel,
			int	command,
			DXB_ARG	arg
		);

/*
 *	When using dxb_rxbuf_read(), a user supplied copy function
 *	can optionally be used instead of the memcpy().  Here is
 *	its prototype.  The "dst" buffer address of the last location
 *	written plus 1 is returned (i.e. the next free location in
 *	"dst").  This is because the function may be called 1 or 2
 *	times to move a whole read request.
 */
typedef void *	(*DXB_COPYFUNC)(
			void	*dst,
			uint8	*src,
			int	len,
			DXB_ARG
		);

/*
 *	The API can optionally provide an automatic circular buffer
 *	for handling unbounded receive data that the user can leave
 *	receive data lying around in.  Here is its structure.
 *
 *	NOTE: the user should not use any internal members of this.
 */
typedef struct
{
	int		in;	/* Producers (rxpoll routine) 'in' offset */
	int		out;	/* Consumers 'out' offset */
	int		size;	/* Size of circular buffer */
	int		mask;	/* size-1 of circular buffer */
	int		free;	/* If true, buffer should be freed on close */
	uint8		*buf;	/* The buffer itself, dynamically allocated */
	DXB_COPYFUNC	copyout;	/* Routine to copy data out buf */
	DXB_ARG		copyout_arg;	/* User arg for above */
} DXB_RXBUF;

/*
 *	The API can optionally provide an automatic circular buffer
 *	for handling bounded receive data that the user can leave
 *	receive data lying around in.  Here are its structures.
 *
 *	NOTE: the user should not use any internal members of these.
 */
typedef struct
{
	uint8	*s;
	uint8	*e;
} DXBSE;

typedef struct
{
	int		in;	/* Producers (rxpoll routine) 'in' offset */
	int		out;	/* Consumers 'out' offset */
	int		size;	/* Size of circular buffer */
	int		mask;	/* size-1 of circular buffer */
	int		free;	/* If true, buffer should be freed on close */
	uint8		*buf;	/* The buffer itself, dynamically allocated */
				/* FIXME: I really think this should be a */
				/* DXBBUF */

	DXBSE		ctrl;	/* Ptrs to start and end of current ctrl part */
	DXBSE		data;	/* Ptrs to start and end of current data part */
	/* msg holds the header of the current message */
	uint8		msg[(DXB_MSG_DATA_SIZE > DXB_MSG_CDATA_SIZE)
			   ? DXB_MSG_DATA_SIZE : DXB_MSG_CDATA_SIZE ] ;

	int		msgvalid ; /* set if msg union is valid */
	

} DXB_DXBUF;


/*
 *	The per channel data
 */
typedef struct
{
	int		credits;

	int		credits_owed;

	int		credit_return_level;

	int		my_fixed_cost;
	int		his_fixed_cost;

	int		credit_cb_on_space;
	int		need_credit_cb;

	int		nak_error_code;

	DXB_OPENACK_CB	openack_cb;
	DXB_ARG		openack_cb_arg;

	DXB_DATA_CB	data_cb;
	DXB_ARG		data_cb_arg;

	DXB_CDATA_CB	cdata_cb;
	DXB_ARG		cdata_cb_arg;

	DXB_CLOSE_CB	close_cb;
	DXB_ARG		close_cb_arg;

	DXB_CLOSEACK_CB	closeack_cb;
	DXB_ARG		closeack_cb_arg;

	DXB_CREDIT_CB	credit_cb;
	DXB_ARG		credit_cb_arg;

	DXB_PUSH_CB	push_cb;
	DXB_ARG		push_cb_arg;

	DXB_PUSHACK_CB	pushack_cb;
	DXB_ARG		pushack_cb_arg;

	DXB_POP_CB	pop_cb;
	DXB_ARG		pop_cb_arg;

	DXB_POPACK_CB	popack_cb;
	DXB_ARG		popack_cb_arg;

	DXB_LINK_CB	link_cb;
	DXB_ARG		link_cb_arg;

	DXB_LINKACK_CB	linkack_cb;
	DXB_ARG		linkack_cb_arg;

	DXB_UNLINK_CB	unlink_cb;
	DXB_ARG		unlink_cb_arg;

	DXB_UNLINKACK_CB unlinkack_cb;
	DXB_ARG		unlinkack_cb_arg;

	DXB_FLUSH_CB	flush_cb;
	DXB_ARG		flush_cb_arg;

	DXB_FLUSHACK_CB	flushack_cb;
	DXB_ARG		flushack_cb_arg;

	DXB_IOCTL_CB	ioctl_cb;
	DXB_ARG		ioctl_cb_arg;

	DXB_IOCTLACK_CB	ioctlack_cb;
	DXB_ARG		ioctlack_cb_arg;

	DXB_CANCEL_CB	cancel_cb;
	DXB_ARG		cancel_cb_arg;

	/*
	 *	Stuff needed for rxbuf option
	 */
	DXB_RXBUF	*rxbuf;

	/*
	 *	Stuff needed for dxbuf option
	 */
	DXB_DXBUF	*dxbuf;

} DXBCHAN;


/*
 *	This is the definition of a DXB instance.
 *
 *	Note that one of these structures exists on each side of
 *	a DXB instance.
 *
 */
struct dxb
{
	/*
	 *	The two queues
	 */
	DXBQ	host2adapter_q;

	DXBQ	adapter2host_q;

	/*
	 *	A copy of the initialization data
	 */
	DXBPHYS	dxbphys;

	/*
	 *	This is a flag that indicates whether this instance of DXB
	 *	is the side that initiates opens.  This is needed by
	 *	the API code so that it can be either an openside or the
	 *	slave side.
	 */
	int	openside;

	/*
	 *	If there is an error in the poller, this flag will get
	 *	set and the poller will stop processing messages.
	 */
	int	poll_error;

	/*
	 *	For internal convenience, pointers to *our* inq and outd
	 */
	DXBQ	*inq;
	DXBQ	*outq;

	/*
	 *	For fair scheduling when returning credit messages,
	 *	keep a variable which rotates the first channel
	 *	we check.  Also, to improve performance, keep a
	 *	variable that indicates the biggest DXB channel
	 *	number currently open.
	 */
	int	first_channel;
	int	min_channel;
	int	max_channel;

	/*
	 *	The callback routine when an open message is received
	 */
	DXB_OPEN_CB	open_cb;
	DXB_ARG		open_cb_arg;

	/*
	 *	Private data pointer supplied by user, for use
	 *	in user supplied macros.
	 */
	DXB_ARG	userdata;
	DXB_ARG	syncdata;

	/*
	 *	A bitmap to indicate which channels are opened
	 */
	uint32	openmap[DXB_MAXCHAN/32];

#ifdef	DXB_MP
	MPLockPtr	inlockptr;
	MPLock		inlock;
	MPLockPtr	openmap_lockptr;
	MPLock		openmap_lock;
	DXB_MPSTATE	mpstate[DXB_MP];
#endif

	/*
	 *	Per channel data
	 */
	DXBCHAN	chan[DXB_MAXCHAN];
};


#ifndef	DXB_MP
#define	OUT_TMP(q)	(q->out_tmp)
#define	OUT(q)	(q->out)
#define	HIGHPRI(q)	(q->highpri)
#define	CHANNEL(q)	(q->channel)
#define	DATA_LEN(q)	(q->data_len)
#define	DATA_CNT(q)	(q->data_cnt)
#define	CTRL_LEN(q)	(q->ctrl_len)
#define	CTRL_CNT(q)	(q->ctrl_cnt)
#endif


/**************************************************************************/
/*
 *	API function prototypes
 *
 *	NOTE: all API calls or sequences of calls must be LOCKed by the
 *	caller to avoid critical regions.  LOCK(), UNLOCK(), and BLOCK()
 *	are pseudo-code for the user's operating system functions to do this.
 *
 *	The function prototypes have been categorized below for your
 *	convenience:
 *		- initialization
 *		- open/close
 *		- data sending
 *		- ioctl
 *		- STREAMs support
 *		- receive side and poller
 */
/**************************************************************************/

/************************************************************************/
/*									*/
/*	API Initialization functions					*/
/*									*/
/************************************************************************/

/*
 *      dxb_init:       Initialize a DXB instance
 *
 *      This function must be called before any DXB activity can take place.
 *
 *      The number of pages and the size of each page specifies
 *      the size of the two buffers we will allocate.  Yes, both the
 *      host to adapter and adapter to host queues are set to the
 *      same size.
 *
 *      The "openside" boolean specifies if we are the side
 *      that opens connections (host==1==DXB_OPENSIDE) or the
 *      slave side (adapter==0==DXB_SLAVESIDE).
 *
 *      Returns 0 on success, DXB_ENOMEM on allocation failure.
 *
 *      Typical host usage:
 *              DXB     Dxb;
 *              rc = dxb_init(&Dxb, 32, 4096, DXB_OPENSIDE);
 *
 *      Typical adapter usage:
 *              DXB     Dxb;
 *              rc = dxb_init(&Dxb, 1, 32*4096, DXB_SLAVESIDE);
 *
 *              NOTE: the adapter side usually uses the info in the phys
 *              structure to set the size of its buffer the same as the
 *              host.  So the call is more like:
 *
 *              rc = dxb_init(&Dxb, 1, dxbphys.host2adapter_q.size,
 *                              DXB_SLAVESIDE);
 */
extern int	dxb_init(
			DXB	*dxb,
			int	npages,
			int	pagesize,
			int	openside
		);

			/*
			 *	"openside" values for readability only
			 */
			#define	DXB_SLAVESIDE	0
			#define	DXB_OPENSIDE	1


/*
 *      dxb_uninit:	Uninitialize a DXB instance.  Free resources.
 */
extern void	dxb_uninit(
			DXB  *dxb
		);

/*
 *	dxb_phys_copy:	Make a (potentially byteswapped) copy of the 
 *	DXBPHYS structure.
 */
extern	void	dxb_phys_copy(
			DXBPHYS *dst, 	
			const DXBPHYS *src, 
			uint swap
		);

/*
 *      dxb_set_phys:	Fill in the phys structure
 *
 *      This routine is called by the host, after dxb_init(), and
 *      just before firing up the firmware.  This prepares a DXBPHYS
 *      structure with physical buffer addresses which the host
 *      then communicates to the firmware (see triple boot protocol)
 *      using a global pointer in shared memory.
 */
extern void	dxb_set_phys(
			DXB *dxb,
			DXBPHYS *phys
		);

/*
 *	dxb_dxbbuf_parms:
 *
 *	Given a DXBBUF buf pointer with buf->npages and buf->pagesize
 *	filled in, compute the buf->pagemask, buf->pageshift, buf->size,
 *	buf->mask, and buf->mask4 paramters which are used in the DXB
 *	library for performance reasons.
 *
 *	This routine is exposed external to the DXB library so that it
 *	may be called by the adapter side when it receives the physical
 *	buffer information at initialization time.
 */
extern void	dxb_dxbbuf_parms(DXBBUF *buf);

/************************************************************************/
/*									*/
/*	API Open/close functions					*/
/*									*/
/************************************************************************/

/*
 *      dxb_open:	Enqueue an open message
 *
 *      The "devid" is a NULL terminated character string specifying
 *      the protocol/module/whatever to be opened.  The "fixed_cost"
 *      is the cost of sending a message that we want the other side
 *      to use in computing credit requirements when sending to us.
 *      The "data" and "data_len" are additional data to be sent with
 *      the open message.
 *
 *      This routine is called only by the "openside" (host).  The
 *      slave side will get a a callback when it receives this message,
 *      which it can handle as it pleases, but typically by sending
 *      back an open acknowledgement.  The "openside" should call
 *      set_openack_cb() immediately after calling this routine so
 *      that it can get a callback when the open is completed.
 *
 *      Returns a channel number (0-255) on success.  Returns
 *      DXB_EMCHAN if all channels are in use.  Returns DXB_ENOSPACE
 *      if there isn't space to send the message right now (try later).
 *
 *      Typical use:
 *              uint32  minor = 42;
 *              for (;;)
 *              {
 *                      DXB_LOCK(); 
 *                      dxbchan = dxb_open(&Dxb, "SERIAL", 0,
 *                                              &minor, sizeof(minor));
 *                      if (dxbchan >= 0) break;
 *                      DXB_UNLOCK();
 *                      BLOCK();
 *              }
 *              
 *              dxb_set_openack_cb(&Dxb, dxbchan, ack_cb, minor);
 *              DXB_UNLOCK();
 *              BLOCK();        // Until ack_cb() gets called
 * 
 */
extern int	dxb_open(
			DXB	*dxb,
			char	*devid,
			int	fixed_cost,
			void	*data,
			int	data_len
		);

/*
 *      dxb_openack:	Enqueue an open acknowledgment message
 *
 *      This routine sends the response to an open message that has been
 *      received.  It is called only by the slave (adapter).  The "fixed_cost"
 *      is the cost of sending a message that we want the other side
 *      to use in computing credit requirements when sending to us.
 *      The "error_code" should be 0 if the open has been performed,
 *      else anything non-zero is an error that is sent to the host.
 *
 *      Returns 0 on success. Returns DXB_ENOSPACE if there isn't space
 *      to send the message right now (try later).
 *
 *      A typical use is in a DXB_OPEN_CB callback routine (note that
 *      in a typical implementation, LOCKs are already held by now):
 *
 *              rc = dxb_openack(&Dxb, channel, 0, 0);
 *              if (rc != 0)
 *                      must defer ack until later by some mechanism
 *
 */
extern int	dxb_openack(
			DXB	*dxb,
			int	channel,
			int	fixed_cost,
			int	error_code
		);

/*
 *      dxb_opennak:	Enqueue a failed open acknowledgment message
 *
 *	This routine is used to queue a negative acknowledgement to
 *	an open request.  The "error_code" must be non-zero.  This
 *	routine is different from dxb_openack in that if the message
 *	cannot be sent, then the dxb library will automatically attempt
 *	to send it each time dxb_rxpoll() is called.
 *
 *	Returns DXB_EPERM if called by the openside, DXB_EINVAL if called
 *	with a zero "error_code", otherwise returns 0.
 *
 *	The reason this function exists is as a convenience for users.
 *	When NAKing an open, the caller typically wants to retain no
 *	state information or data structures for the open.  So as a
 *	convenience the library maintains the state for this one case only.
 *
 */
extern int	dxb_opennak(
			DXB	*dxb,
			int	channel,
			int	error_code
		);

/*
 *      dxb_close:	Enqueue a close message
 *
 *      "channel" is the DXB channel to close.
 *      Called by "openside" (host) only.
 *
 *      Returns 0 on success, DXB_ENOSPACE if there is no room for message.
 *
 *      Typical use:
 *              for (;;)
 *              {
 *                      DXB_LOCK();
 *                      rc = dxb_close(&Dxb, channel);
 *                      DXB_UNLOCK();
 *                      if (rc == 0) break;
 *                      BLOCK();
 *              }
 */
extern int	dxb_close(
			DXB	*dxb,
			int	channel
		);

/*
 *      dxb_closeack:   Enqueue a close acknowledgement message
 *
 *      This routine sends the response to an open message that has
 *      been received.  It is called only by the slave (adapter).
 *
 *      Returns 0 on success. Returns DXB_ENOSPACE if there isn't space
 *      to send the message right now (try later).
 *
 *      A typical use is in a DXB_CLOSE_CB callback routine (note that
 *      in a typical implementation, LOCKs are already held by now):
 *
 *              rc = dxb_closeack(&Dxb, channel, 0, 0);
 *              if (rc != 0)
 *                      must defer ack until later by some mechanism
 */
extern int	dxb_closeack(
			DXB	*dxb,
			int	channel
		);

/************************************************************************/
/*									*/
/*	API Data message sending functions				*/
/*									*/
/************************************************************************/

/*
 *      dxb_data_payload_space:
 *
 *      Return the number of bytes available for sending DATA
 *      message payload on a particular "channel".  Typically
 *      used by ASYNC channels where the data is unbounded.
 */
extern int	dxb_data_payload_space(
			DXB	*dxb,
			int	channel
		);

/*
 *      dxb_cdata_payload_space:
 *
 *      Return the number of bytes available for sending CDATA
 *      message payload on a particular "channel".  Typically
 *      used by ASYNC channels where the data is unbounded.
 *      "ctrl_len" is the amount of space needed by the control
 *      portion.  "highpri" is 1 if this message is not subject
 *      to flow control.
 */
extern int	dxb_cdata_payload_space(
			DXB	*dxb,
			int	channel,
			int	ctrl_len,
			int	highpri
		);

/*
 *      dxb_space_avail:
 *
 *      Return 0 if there is space available to send a message on a
 *      "channel" of the given "ctrl_len" and "data_len".  "highpri"
 *      is 1 if the message is not subject to flow control.
 *      Else return ENOSPACE or ENOCRED.
 *
 *      Typically used by non-ASYNC channels where the data is
 *      a bounded message (packet).
 */
extern int	dxb_space_avail(
			DXB	*dxb,
			int	channel,
			int	ctrl_len,
			int	data_len,
			int	highpri
		);

/*
 *	dxb_queue_space:
 *
 *	Return the amount of free space in the DXB queue itself.
 *
 *	This routine returns a pessimistic amount, intentionally.
 *	Its main use is to determine, after a call to *_payload_space()
 *	returns 0, if the reason is due to queue overload (as opposed
 *	to no available credits).
 */
extern int	dxb_queue_space(DXB *dxb);

/*
 *      dxb_data:       Enqueue a DATA message header.
 *
 *      "channel" is the channel number, "data_len" is the length in
 *      bytes of the data portion that will be enqueued using
 *      dxb_write_data_block().  Space for the message must have
 *      been previously guaranteed using dxb_space_avail(), or by
 *      using dxb_data_payload_space().
 *
 *      Returns 0 on success, otherwise an error (i.e. illegal channel).
 *
 *      Typical use of API for sending a DATA message:
 *
 *      for (;;)
 *      {
 *              DXB_LOCK();
 *              if (dxb_space_avail(dxb, chan, 0, data_len, 0) == DXB_OK)
 *                      break;
 *              DXB_UNLOCK();
 *              BLOCK();
 *      }
 *      dxb_data(dxb, chan, data_len);
 *      dxb_write_data_block(dxb, data, data_len);
 *      dxb_write_done(dxb);
 *      DXB_UNLOCK();
 */
extern int	dxb_data(
			DXB	*dxb,
			int	channel,
			int	data_len
		);

/*
 *      dxb_cdata:      Enqueue a CDATA message header
 *
 *      "channel" is the channel number, "highpri" is 1 if this message
 *      is not subject to flow control. "ctrl_len" and "data_len" are
 *      the length in bytes of the control and data portions that will
 *      be enqueued using dxb_write_data_block().  Space for the message
 *      must have been previously guaranteed using dxb_space_avail(), or by
 *      using dxb_data_payload_space().
 *
 *      Returns 0 on success, otherwise an error (i.e. illegal channel).
 *
 *      Typical use of API for sending a CDATA message:
 *
 *      for (;;)
 *      {
 *              DXB_LOCK();
 *              if (dxb_space_avail(dxb, chan, ctrl_len, data_len, 0) == DXB_OK)
 *                      break;
 *              DXB_UNLOCK();
 *              BLOCK();
 *      }
 *      dxb_cdata(dxb, chan, 0, ctrl_len, data_len);
 *      dxb_write_ctrl_block(dxb, ctrl, ctrl_len);
 *      dxb_write_data_block(dxb, data, data_len);
 *      dxb_write_done(dxb);
 *      DXB_UNLOCK();
 */
extern int	dxb_cdata(
			DXB	*dxb,
			int	channel,
			int	highpri,
			int	ctrl_len,
			int	data_len
		);

/*
 *      dxb_write_data_block:
 *
 *      Enqueue a block of data.  This routine may be called more than
 *      once if it is desired to queue the data for a message in several
 *      pieces, such as from fragmented STREAMs mblk's.
 *
 *      Returns 0 if AOK, returns DXB_EINVAL if too much data has been queued.
 */
extern int	dxb_write_data_block(
			DXB	*dxb,
			const void *data,
			int	data_len
		);

extern void	*dxb_peek_data_write(
			DXB	*dxb,
			int	*max
		);

/*
 *      dxb_write_ctrl_block:
 *
 *      Enqueue a block of control data.  This routine may be called more than
 *      once if it is desired to queue the data for a message in several
 *      pieces, such as from fragmented STREAMs mblk's.
 *
 *      Returns 0 if AOK, returns DXB_EINVAL if too much data has been queued.
 */
extern int	dxb_write_ctrl_block(
			DXB	*dxb,
			const void *ctrl,
			int	ctrl_len
		);

extern void	*dxb_peek_ctrl_write(
			DXB	*dxb,
			int	*max
		);

extern int	dxb_peek_wconfirm(
			DXB	*dxb,
			int	count
		);

/*
 *      dxb_write_done:
 *
 *      Advance in pointer, marking the end of a data message
 *
 *      Returns 0 if everything is AOK.  Returns DXB_EINVAL and does
 *      not send the message if the amount of control or data information
 *      queued is not the same as what was promised.
 */
extern int	dxb_write_done(
			DXB	*dxb
		);


/************************************************************************/
/*									*/
/*	API ioctl message sending functions				*/
/*									*/
/************************************************************************/

/*
 *      dxb_ioctl:      Enqueue an ioctl message
 *
 *      "channel" is the channel number for the message.  "command"
 *      is the ioctl command number.  "data_len" is the length of
 *      data to be sent with the command.  Data is passed in a manner
 *      similar to dxb_data() messages, but using readability macros
 *      dxb_ioctl_space_avail() and dxb_write_ioctl_block().
 *
 *      Returns 0 on success, otherwise an error (i.e. illegal channel).
 *
 *      Typical use of API for sending an ioctl message:
 *
 *      for (;;)
 *      {
 *              LOCK();
 *              if (dxb_ioctl_space_avail(dxb, chan, data_len) == DXB_OK)
 *                      break;
 *              UNLOCK();
 *              BLOCK();
 *      }
 *      dxb_ioctl(dxb, chan, command, data_len);
 *      dxb_write_ioctl_block(dxb, data, data_len);
 *      dxb_write_done(dxb);
 *      UNLOCK();
 */
extern int	dxb_ioctl(
			DXB		*dxb,
			int		channel,
			int		command,
			int		data_len
		);

/*
 *      dxb_ioctlack:   Enqueue an ioctl acknowledgement message
 *
 *      "channel" is the channel number for the message.  "command"
 *      is the ioctl command number.  "data_len" is the length of
 *      data to be sent with the command.  "error_code" is the error
 *      indicator (0 is AOK). Data is passed in a manner
 *      similar to dxb_data() messages, but using readability macros
 *      dxb_ioctlack_space_avail() and dxb_write_ioctl_block().
 *
 *      Returns 0 on success, otherwise an error (i.e. illegal channel).
 *
 *      Typical use of API for sending an ioctl message:
 *
 *      for (;;)
 *      {
 *              DXB_LOCK();
 *              if (dxb_ioctlack_space_avail(dxb, chan, data_len) == DXB_OK)
 *                      break;
 *              DXB_UNLOCK();
 *              BLOCK();
 *      }
 *      dxb_ioctlack(dxb, chan, command, data_len);
 *      dxb_write_ioctl_block(dxb, data, data_len);
 *      dxb_write_done(dxb);
 *      DXB_UNLOCK();
 */
extern int	dxb_ioctlack(
			DXB		*dxb,
			int		channel,
			int		command,
			int		data_len,
			int		error_code
		);

/*
 *	dxb_write_ioctl_block:
 *
 *	Another name for dxb_write_data_block(), provided for readbility.
 */
#define	dxb_write_ioctl_block	dxb_write_data_block

/*
 *	dxb_ioctl_space_avail:
 *	dxb_ioctlack_space_avail:
 *
 *	Versions of dxb_space_avail() for ioctl's that know about the
 *	size of the control portion of an ioctl message.
 */
#define	dxb_ioctl_space_avail(DXB, CHAN, DATALEN) \
		dxb_space_avail(DXB, CHAN, 0, DATALEN, 1)

#define	dxb_ioctlack_space_avail(DXB, CHAN, DATALEN) \
		dxb_space_avail(DXB, CHAN, 4, DATALEN, 1)

/*
 *      dxb_cancel:
 *
 *      Enqueue a cancel ioctl message for "channel".  "command" is
 *      the ioctl being canceled.
 */
extern int	dxb_cancel(
			DXB	*dxb,
			int	channel,
			int	command
		);

/*
 *	dxb_cancelack:
 *
 *      Enqueue a cancel ioctl ack message for "channel".  "command" is
 *      the ioctl being canceled.
 */
#define	dxb_cancelack(DXB, CHAN, CMD)	dxb_cancel(DXB, CHAN, CMD)
			

/************************************************************************/
/*									*/
/*	API STREAMS support functions					*/
/*									*/
/************************************************************************/
extern int	dxb_link(
			DXB	*dxb,
			int	channel,
			int	other_channel
		);

extern int	dxb_linkack(
			DXB	*dxb,
			int	channel,
			int	mux_id,
			int	error_code
		);

extern int	dxb_unlink(
			DXB	*dxb,
			int	channel,
			int	mux_id
		);

extern int	dxb_unlinkack(
			DXB	*dxb,
			int	channel,
			int	mux_id,
			int	error_code
		);

extern int	dxb_push(
			DXB		*dxb,
			int		channel,
			const char	*module_id,
			const void	*data,
			int		data_len
		);

extern int	dxb_pushack(
			DXB	*dxb,
			int	channel,
			int	error_code
		);

extern int	dxb_pop(
			DXB	*dxb,
			int	channel
		);

extern int	dxb_popack(
			DXB	*dxb,
			int	channel,
			int	error_code
		);

extern int	dxb_flush(
			DXB	*dxb,
			int	channel,
			int	flush_read,
			int	flush_write
		);

extern int	dxb_flushack(
			DXB	*dxb,
			int	channel,
			int	error_code
		);

/************************************************************************/
/*									*/
/*	API Receive side functions and poller				*/
/*									*/
/************************************************************************/
/*
 *      dxb_read_data_block:
 *
 *      Read part or all of the control or data portion of a message.
 *
 *      "first" is normally set to "1", which means we are reading the
 *      entire control or data portion.  If the caller wants to read
 *      either portion in pieces (for example, to dump the data into
 *      fragmented STREAMS buffers), then "first" should be a "1" for
 *      the first read and "0" for subsequent reads of that portion.
 *
 *      "buf_size" is the size of the buffer to put it in.  "data_len"
 *      is the amount of data to be handled.  If bufsize is less than
 *      data_len, only bufsize bytes will be moved, and the rest will
 *      be skipped.
 *
 *      Always returns 0; Honest.
 *
 *      Typical use is from DXB_DATA_CB, DXB_CDATA_CB, DXB_IOCTL_CB, 
 *      or DXB_IOCTLACK_CB callback routines.  The following are
 *	rather simplistics examples;  real code would put the data
 *	into, say, a STREAMs mblk, and wouldn't return the credits
 *	until the mblk had been completely processed.
 *
 *      int
 *      data_cb(DXB *dxb, int channel, int data_len, DXB_ARG action)
 *      {
 *              char    data[512];
 *
 *              dxb_read_data_block(dxb, data, sizeof(data), data_len);
 *              dxb_read_done(dxb);
 *              // ... do something with data[] ...
 *              dxb_return_fixed_cost(dxb, channel, 1);
 *              dxb_return_credits(dxb, channel, data_len);
 *              return (0);
 *      }
 *
 *      int     
 *      cdata_cb(DXB *dxb, int channel, int priority,
 *                      int ctrl_len, int data_len, DXB_ARG action)
 *      {
 *              char    ctrl[512];
 *              char    data[512];
 *       
 *              dxb_read_data_block(dxb, ctrl, sizeof(ctrl), ctrl_len);
 *              dxb_read_data_block(dxb, data, sizeof(data), data_len);
 *              dxb_read_done(dxb); 
 *              // ... do something with ctrl[], data[] ...
 *              if (!priority)
 *              {
 *                      dxb_return_fixed_cost(dxb, channel, 1);
 *                      dxb_return_credits(dxb, channel, ctrl_len + data_len);
 *              }
 *              return (0);
 *      }
 */
extern int	dxb_read_ctrl_block(
			DXB	*dxb,
			void	*buf,
			int	buf_size,
			int	ctrl_len
		);

extern void	*dxb_peek_ctrl_read(
			DXB	*dxb,
			int	*max
		);

extern int	dxb_read_data_block(
			DXB	*dxb,
			void	*buf,
			int	buf_size,
			int	data_len
		);

extern void	*dxb_peek_data_read(
			DXB	*dxb,
			int	*max
		);

extern int	dxb_peek_rconfirm(
			DXB	*dxb,
			int	count
		);

/*
 *      dxb_read_data_done:
 *
 *      Indicate that the caller has read all of the data and it is safe
 *      to advance the out pointer.
 *
 *      Always returns 0; Honest.
 */
extern int	dxb_read_done(
			DXB	*dxb
		);

/*
 *      dxb_return_credits:
 *
 *      This routine is called to return payload credits for a channel.
 *      "credits" should be the number of control and data message
 *      bytes being credited.  Fixed cost credit returns are handled by
 *      dxb_return_fixed_cost().
 *
 *      This routine is typically called for two reasons:
 *      1) to initially supply credits after calling dxb_open()/dxb_openack() 
 *      2) to replenish credits after processing a received message.
 *
 *      Depending on the setting of dxb_set_credit_return_level(), this
 *      routine may or may not generate an actual DXB message at the
 *      time that it is called.
 *
 *      This routine returns 0 always.  Honest.
 *
 *      Typical use for the first case is:
 *              dxb_return_credits(dxb, channel, 4096);
 *              dxb_return_fixed_cost(dxb, channel, 100);
 *
 *      Typical use for the second case is:
 *              dxb_return_credits(dxb, channel, ctrl_len + data_len);
 *              dxb_return_fixed_cost(dxb, channel, 1);
 *
 */
extern int	dxb_return_credits(
			DXB	*dxb,
			int	channel,
			int	credits
		);

/*
 *      dxb_return_fixed_cost:
 *
 *      Return the fixed credits for a channel.  This is an additional
 *      abstraction on top of the previous routine.  "Count" multiples
 *      of the fixed cost are returned.  This routine should be called
 *      once per data message received and processed.
 *
 *      This routine returns 0 always.  Honest.
 *
 *      NOTE: this abstraction acknowledges the fact that a DXB instance
 *      may be prepared to handle N bytes of received data and/or M
 *      total messages, whichever is lesser.  However, the underlying
 *      DXB implementation doesn't yet understand the difference and
 *      lumps both into a single figure of merit called credits.
 */
extern int	dxb_return_fixed_cost(
			DXB	*dxb,
			int	channel,
			int	count
		);

/*
 *	dxb_credits_avail:
 *
 *	Return the number of bytes available in the port's data
 *              queue.  It does not account for the dxb queue.
 */
extern int dxb_data_credits_avail(
			DXB *dxb, 
			int channel
		);



/*
 *      dxb_poll:
 *
 *      Poll routine.  Must be called at the poll rate, which
 *      is typically every 10 msecs or as fast as you can stand it.
 *      Processes received message and does housekeeping chores
 *      like generation of transmitted credit and nak messages.
 *
 *      This is a convenience routine that simply calls
 *      dxb_rxpoll_all() and dxb_txpoll_all().  Any errors
 *      from those routines are propogated back to the caller.
 *
 *      Some library users may prefer the finer control of directly
 *      calling dxb_rxpoll_all(), dxb_rxpoll_one(), and dxb_txpoll_all().
 *
 *      Typical use:
 *              timer_tick()
 *              {
 *                      DXB_LOCK;
 *                      dxb_poll(&Dxb);
 *                      DXB_UNLOCK;
 *              }
 */
extern int	dxb_poll(
			DXB	*dxb
		);

#define		dxb_rxpoll(DXB) dxb_poll(DXB)	/* Backward compatibility */

#ifndef	DXB_MP
/*
 *	dxb_rxpoll_one:
 *
 *	Processes a single received message.  This routine is
 *	called by dxb_rxpoll_all().  It is exposed to the user
 *	of the DXB library to allow fine control on how many
 *	DXB messages are processed on each poll.
 *
 *      All DXB library callbacks except for the credit_cb are
 *	initiated from this routine.
 *
 *	If an error occurs due to an invalid command recieved, the
 *	routine will return DXB_EINVAL and will stop processing
 *	further messages.
 *
 *	This routine returns 1 if more messages remain to be processed
 *	and 0 if there are no more messages to be processed.
 *
 *	If you chose to use the lower level dxb_rxpoll_one() and
 *	friends, the typical usage is:
 *
 *		for (;;)
 *		{
 *			DXB_LOCK;
 *			rc = dxb_rxpoll_one(dxb);
 *			DXB_UNLOCK;
 *			if (rc <= 0) break;
 *		}
 *		DXB_LOCK;
 *		dxb_txpoll_all();
 *		DXB_UNLOCK;
 *	
 */
extern int	dxb_rxpoll_one(
			DXB	*dxb
		);

/*
 *	dxb_rxpoll_all:
 *
 *	Processes all received messages.  Repeatedly calls
 *	dxb_rxpoll_one() until received message queue is
 *	empty.  Any errors from that routine are propogated
 *	back to the caller.
 */
extern int	dxb_rxpoll_all(
			DXB	*dxb
		);
#endif

/*
 *	dxb_txpoll_all:
 *
 *	Must be called periodically to perform housekeeping
 *	chores on each DXB channel, such as generation of
 *      credit messages and opennak messages.  We also do
 *      the credit_cb callback for credit returns here, so
 *	they are scheduled fairly.
 */
extern void	dxb_txpoll_all(
			DXB	*dxb
		);

/*
 *	dxb_set_credit_return_level:
 *
 *	Set the credit return level for a channel.  If we owe more
 *	than this number of credits after a dxb_return_credits() call,
 *	a return message will be sent immediately.  If the level is set
 *	to zero, then credits will only be returned when dxb_rxpoll()
 *	is called.
 */
DXB_INLINE(
int
dxb_set_credit_return_level(DXB *dxb, int channel, int level),
{
	dxb->chan[channel].credit_return_level = level;
	return (0);
}
)

/*
 *	dxb_set_credit_cb_on_space:
 *
 *	Set a flag to indicate that we want our credit callback
 *	function to be called if space becomes available in the
 *	DXB queue (in addition to when more credits are available).
 *
 *	Note: it is the users responsibility to set this flag
 *	when needed, and to clear it when it is not needed. If
 *	it is left set, then the credit callback function will
 *	likely get called on just about every dxb_poll().
 *
 *	Typical use is to set this flag when you get an ENOSPACE
 *	indication from dxb_space_avail(), and then to clear it
 *	when you get the credit callback.
 */
DXB_INLINE(
void
dxb_set_credit_cb_on_space(DXB *dxb, int channel, int flag),
{
	dxb->chan[channel].credit_cb_on_space = flag;
}
)

/************************************************************************/
/*									*/
/*	API routines to set callback functions.				*/
/*	Inline because they are simple.					*/
/*									*/
/************************************************************************/
DXB_INLINE(
void
dxb_set_open_cb(DXB *dxb, DXB_OPEN_CB cb, DXB_ARG arg),
{
	dxb->open_cb = cb;
	dxb->open_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_openack_cb(DXB *dxb, int channel, DXB_OPENACK_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].openack_cb = cb;
	dxb->chan[channel].openack_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_data_cb(DXB *dxb, int channel, DXB_DATA_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].data_cb = cb;
	dxb->chan[channel].data_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_cdata_cb(DXB *dxb, int channel, DXB_CDATA_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].cdata_cb = cb;
	dxb->chan[channel].cdata_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_close_cb(DXB *dxb, int channel, DXB_CLOSE_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].close_cb = cb;
	dxb->chan[channel].close_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_closeack_cb(DXB *dxb, int channel, DXB_CLOSEACK_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].closeack_cb = cb;
	dxb->chan[channel].closeack_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_credit_cb(DXB *dxb, int channel, DXB_CREDIT_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].credit_cb = cb;
	dxb->chan[channel].credit_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_push_cb(DXB *dxb, int channel, DXB_PUSH_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].push_cb = cb;
	dxb->chan[channel].push_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_pushack_cb(DXB *dxb, int channel, DXB_PUSHACK_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].pushack_cb = cb;
	dxb->chan[channel].pushack_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_pop_cb(DXB *dxb, int channel, DXB_POP_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].pop_cb = cb;
	dxb->chan[channel].pop_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_popack_cb(DXB *dxb, int channel, DXB_POPACK_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].popack_cb = cb;
	dxb->chan[channel].popack_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_link_cb(DXB *dxb, int channel, DXB_LINK_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].link_cb = cb;
	dxb->chan[channel].link_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_linkack_cb(DXB *dxb, int channel, DXB_LINKACK_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].linkack_cb = cb;
	dxb->chan[channel].linkack_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_unlink_cb(DXB *dxb, int channel, DXB_UNLINK_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].unlink_cb = cb;
	dxb->chan[channel].unlink_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_unlinkack_cb(DXB *dxb, int channel, DXB_UNLINKACK_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].unlinkack_cb = cb;
	dxb->chan[channel].unlinkack_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_flush_cb(DXB *dxb, int channel, DXB_FLUSH_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].flush_cb = cb;
	dxb->chan[channel].flush_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_flushack_cb(DXB *dxb, int channel, DXB_FLUSHACK_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].flushack_cb = cb;
	dxb->chan[channel].flushack_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_ioctl_cb(DXB *dxb, int channel, DXB_IOCTL_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].ioctl_cb = cb;
	dxb->chan[channel].ioctl_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_ioctlack_cb(DXB *dxb, int channel, DXB_IOCTLACK_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].ioctlack_cb = cb;
	dxb->chan[channel].ioctlack_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_cancel_cb(DXB *dxb, int channel, DXB_CANCEL_CB cb, DXB_ARG arg),
{
	dxb->chan[channel].cancel_cb = cb;
	dxb->chan[channel].cancel_cb_arg = arg;
}
)

DXB_INLINE(
void
dxb_set_cancelack_cb(DXB *dxb, int channel, DXB_CANCEL_CB cb, DXB_ARG arg),
{
	/* Yes, we overload the cancel_cb */
	dxb->chan[channel].cancel_cb = cb;
	dxb->chan[channel].cancel_cb_arg = arg;
}
)

/************************************************************************/
/*									*/
/*	API functions used when rxbuf option is desired			*/
/*									*/
/************************************************************************/
/*
 *	dxb_rxbuf_open:
 *
 *	Just like dxb_open(), except that the user can ask for the
 *	DXB library to manage a circular buffer for data payload
 *	which the user is free to read at his convenience.  This
 *	is a common need for channels with unbounded data, like
 *	async.
 *
 *	"buf_size" is the size of the circular buffer to be used,
 *	which should have some relationship to the number of credits
 *	which will be supplied on this channel (i.e. at least credits+1).
 *	"buf_size" must be a power of two in length.
 *
 *	"buf" is user supplied storage for the circular buffer.  If
 *	it is NULL, the library will use dxb_malloc() to allocate
 *	the space for a buffer of size "buf_size".
 *
 *	The user will get the usual callback when DATA or CDATA messages
 *	are received.  When a CDATA callback occurs, the user should use
 *	dxb_read_ctrl_block() to read the control portion.  In the callback,
 *	the user can use dxb_read_data_block() or dxb_rxbuf_copyin() to
 *	process the received data message.  Use dxb_rxbuf_copyin()
 *	to cause the data to be copied to the allocated rxbuf for
 *	processing at a later time.  Use dxb_read_data_block() to bypass
 *	the rxbuf circular buffer if you are going to handle the data
 *	immediately.  A mix of the two methods may be optimal for some
 *	drivers to avoid the extra copy which occurs when using rxbuf.
 *
 *	If the user has set a non-zero fixed credit cost, then the DATA
 *	or CDATA callbacks should probably call dxb_return_fixed_cost()
 *	to immediately return the fixed cost.  The cost of the bytes
 *	should be credited only after the user has actually used
 *	dxb_rxbuf_getchar() or dxb_rxbuf_read() to remove the data
 *	from the circular buffer.
 *
 *	Returns the same values as for dxb_open().
 */
extern int      dxb_rxbuf_open(
			DXB	*dxb,
			char	*devid,
			int	fixed_cost,
			void	*data,
			int	data_len,
			uint8	*buf,
			int	buf_size
                );

/*
 *	dxb_rxbuf_openack():
 *
 *	Just like dxb_openack(), but with the rxbuf option as described
 *	under dxb_rxbuf_open() above.
 *
 *	However, you can get an ENOMEM error return if "error_code" is 0.
 *	If this happens, it is the users responsibility to call
 *	dxb_opennak() with the ENOMEM error code.
 */
extern int      dxb_rxbuf_openack(
			DXB	*dxb,
			int	channel,
			int	fixed_cost,
			int	error_code,
			uint8	*buf,
			int	buf_size
                );

/*
 *	dxb_rxbuf_copyin:
 *
 *	Copy the data protion of a DXB message into the rxbuf
 *	circular buffer.  Typically called from the data callback.
 *
 *	Returns DXB_ENOSPACE if there is no space to do the copy,
 *	and the copy isn't done.  Else returns 0;
 */
extern int	dxb_rxbuf_copyin(
			DXB	*dxb,
			int	channel,
			int	data_len
		);

/*
 *	dxb_rxbuf_copyin_with_header:
 *
 *	TODO
 */
extern int	dxb_rxbuf_copyin_with_header(
			DXB	*dxb,
			int	channel,
			int	data_len,
			void	*header,
			int	header_len,
			int	header_align
		);

/*
 *	dxb_rxbuf_getchar:
 *
 *	Get the next byte out of the rx buffer or return -1 if there
 *	are no more bytes in the buffer.
 */
extern int	dxb_rxbuf_getchar(
			DXB	*dxb,
			int	channel
		);

/*
 *	dxb_rxbuf_count:
 *
 *	Return the number of bytes available in the rx buffer
 */
extern int	dxb_rxbuf_count(
			DXB	*dxb,
			int	channel
		);

/*
 *	dxb_rxbuf_read:
 *
 *	Read data out of the rxbuf circular buffer for "channel".
 *	Put it into "buf".  Limit read to "buf_len" bytes.
 *
 *	Returns the number of bytes actually copied.
 *
 *	If the copyout function is not set, 1 or 2 calls to memcpy
 *	are used to move the data.  Else, the copyout function is
 *	called 1 or 2 times to move the data.
 */
extern int	dxb_rxbuf_read(
			DXB	*dxb,
			int	channel,
			void	*buf,
			int	buf_len
		);

/*
 *	dxb_rxbuf_flush:
 *
 *	Flush all data out of the rx buffer.  Return the number of bytes
 *	that were flushed.
 */
extern int	dxb_rxbuf_flush(
			DXB	*dxb,
			int	channel
		);

/*
 *	dxb_rxbuf_set_copyout:
 *
 *	Set the copyout function used to copy data out of the rx buffer.
 *	If "copyout" is NULL, then memcpy() is used to copy the data.
 *	You would use a copyout function if you wanted to massage the
 *	data as it was copied to some other place.
 */
extern int	dxb_rxbuf_set_copyout(
			DXB		*dxb,
			int		channel,
			DXB_COPYFUNC	copyout,
			DXB_ARG		arg
		);

/************************************************************************/
/*									*/
/*	API functions used when dxbuf option is desired			*/
/*									*/
/************************************************************************/

/*
 *	NOTE: I am not entirely happy with the dxbuf API, and it is subject
 *	to change.
 */

/*
 *	dxb_dxbuf_open(), dxb_dxbuf_openack()
 *	
 *	Like dxb_rxbuf_open{*}(), except:
 *
 *	1) A permanent fixed cost of 14 (largest DXB message header plus
 *	   two potential alignments) is automatic. "fixed_cost" is any
 *	   additional cost the user wants to add to any messages sent to us.
 *
 *	2) "buf_size" is the size in bytes of the buffer "buf" supplied,
 *	   or the buffer that is dxb_malloc'ed if "buf" is NULL.  One
 *	   less than that is the credits that are supplied.  There are
 *	   no restrictions, per se, in the library on the size of this
 *	   buffer.
 *
 *	3) You still get the normal callback on every DXB message received.
 *	   This allows you to process messages with no cost (such as
 *	   link, push, pop, etc.) as well as priority cdata messages
 *	   immediately.  These cannot be stored in the local DXB
 *	   circular buffer -- since they have no cost they could
 *	   overflow the storage provided.  Obviously, this limits
 *	   the design of protocols that can use this buffering feature.
 *
 *	4) If you get an ioctl, data, or cdata (with normal priority)
 *	   callback, you need to call dxb_dxbuf_copyin() to copy it
 *	   to the per channel buffer.  The library doesn't do it
 *	   for you automatically because there may be an opportunity
 *	   for you to optimize the case where the dxbuf is empty.
 *	   This way, you have the control to do that, if desired.
 *
 *	5) The actual number of credits sent to the "openside" will
 *	   be buf_size - 4.
 */
extern int      dxb_dxbuf_open(
                        DXB     *dxb,
                        char    *devid,
                        int     fixed_cost,
                        void    *data,
                        int     data_len,
                        uint8   *buf,
                        int     buf_size
                );

extern int      dxb_dxbuf_openack(
                        DXB     *dxb,
                        int     channel,
			int	fixed_cost,
                        int     error_code,
                        uint8   *buf,
                        int     buf_size
                );

/*
 *      dxb_dxbuf_copyin:
 *
 *	Copy the entire *current DXB message* from the main DXB
 *	circular buffer to the per channel circular buffer (for
 *	channels opened with the dxbuf option).  This defers the
 *	message for later processing.
 *
 *	Typically called from the users data, cdata, or ioctl callbacks
 *	(which themselves were called from dxb_rxpoll_one()).  This
 *	routine should be called only on the first callback for a
 *	DXB message when (channel & DXB_DXBUF_CB) == 0 in the callback
 *	routine.
 *
 *      Returns DXB_ENOSPACE if there is no space to do the copy,
 *      and the copy isn't done.  Returns DXB_EINVAL if a copy of
 *	an illegal message type (such as a link message) is attempted.
 *	Else returns 0;
 *
 *	NOTE: if you get DXB_ENOSPACE, something has gone wrong with
 *	the credits mechanism.  This error indicates a fault in your
 *	code.
 */
extern int      dxb_dxbuf_copyin(
                        DXB     *dxb,
                        int     channel
                );

/*
 *      dxb_dxbuf_poll_one:
 *
 *      Processes a single received message from the per-channel
 *	dxb buffer (for channels opened with dxbuf option).  This
 *	routine must be called periodically to process messages
 *	from the buffer which have accumulated.
 *
 *      If an error occurs due to an invalid command recieved, the
 *      routine will return DXB_EINVAL and will stop processing
 *      further messages.
 *
 *	This routine returns 0 if there are no more messages to be
 *	processed, or if the current message was incompletely processed
 *	and the same message will be processed on the next call to
 *	dxb_dxbuf_poll_one.  Otherwise, 1 is returned.  The intent is:
 *	if 1 is returned, there is more usefull work to do right now;
 *	if 0 is returned, there is no usefull work at the moment (either
 *	waiting for input, or blocked on output).
 *
 *	This routine generates a 2nd (or possibly 3rd or Nth) callback
 *	for each DXB message.  To save space, we overload the data and cdata
 *	callbacks by ORing 'DXB_DXBUF_CB' into the channel number.
 *
 *      The typical usage is:
 *
 *              for (;;)
 *              {
 *                      LOCK;
 *                      rc = dxb_dxbuf_poll_one(dxb, channel);
 *                      UNLOCK;
 *                      if (rc <= 0) break;
 *              }
 *      
 */
extern int      dxb_dxbuf_poll_one(
                        DXB     *dxb,
			int	channel
                );

/*
 *	dxb_dxbuf_data_locations:
 *	dxb_dxbuf_ctrl_locations:
 *
 *	Return the location(s) of the N pieces of the data or ctrl portion
 *	of the current DXB data or cdata message in the per channel dxbuf.
 *
 *	There could be multiple pieces because of buffer wrap and/or
 *	buffer fragmentation.  The piece information is returned in
 *	an array which the user allocates.  Currently, there is no
 *	mechanism in this library to allocate a fragmented buffer,
 *	so the user should always allocate an array of 2 DXB_PIECE's.
 *	
 */
typedef struct
{
	int		len;	/* Length of this piece */
	uint8		*piece; /* Start of this piece */
} DXB_PIECE;

extern int      dxb_dxbuf_data_locations(
                        DXB     	*dxb,
                        int     	channel,
			int		*npieces,
			DXB_PIECE	*pieces
                );

extern int      dxb_dxbuf_ctrl_locations(
                        DXB     	*dxb,
                        int     	channel,
			int		*npieces,
			DXB_PIECE	*pieces
                );

/*
 *	dxb_dxbuf_advance
 *
 *	Indicate to the library how much of the control and/or
 *	data portions of the current dxbuf DXB message have been
 *	consumed by the caller.  If ctrlp is NULL, it indicates
 *	that the entire control portion has been consumed.  If
 *	datap is NULL, it indicates that the entire data portion
 *	has been consumed.  If either is not NULL, they indicate
 *	the next location in the message that needs to be consumed.
 *	In this case, the library will remember this and you will
 *	get another callback for this message the next time that
 *	dxb_dxbuf_poll_one() is called.
 */
extern int      dxb_dxbuf_advance(
                        DXB     *dxb,
                        int     channel,
			void	*ctrlp,
			void	*datap
		);


/*
 *	dxb_dxbuf_int_advance
 *
 *	Same function as dxb_dxbuf_advance, except advances by
 *	count, rather than pointers.
 *	If ctrl_cnt is -1, it indicates the entire control
 *	portion has been consumed.  If data_cnt is -1, it indicates
 *	that the entire data portion has been consumed.  If either
 *	is not -1, they indicate how far to advance through the
 *	message on the respective (control or data) portion, eventually
 *	leading	to another callback for the rest of the message.
 */
extern int	dxb_dxbuf_int_advance(
                        DXB     *dxb,
                        int     channel,
			int	ctrl_cnt,
			int	data_cnt
		);

/*
 *	dxb_dxbuf_read_ctrl_block:
 *	dxb_dxbuf_read_data_block:
 *
 *	Like dxb_read_*_block(), but takes its data from the
 *	per channel circular buffer.  These are use in lieu
 *	of dxb_dxbuf_*_locations().
 *
 *	It is possible and allowed to mix both calls, though.
 *	The typical use would be to *read* the control portion,
 *	and *get the locations* of the data portion.
 */
extern int	dxb_dxbuf_read_ctrl_block(
			DXB	*dxb,
			int	channel,
			void	*buf,
			int	buf_size,
			int	ctrl_len
		);

extern int	dxb_dxbuf_read_data_block(
			DXB	*dxb,
			int	channel,
			void	*buf,
			int	buf_size,
			int	data_len
		);

/*
 *	dxb_dxbuf_empty:
 *
 *	Returns true if there is no data queued in dxbuf
 */
extern int	dxb_dxbuf_empty(
			DXB *dxb, 
			int channel
		);

/************************************************************************/
/*									*/
/*	Miscellaneous							*/
/*									*/
/************************************************************************/
/*
 *	API function call error returns.  All errors are guaranteed to
 *	be negative numbers.
 */
#ifndef DXB_OK
#define	DXB_OK		0	/* Everything went AOK, for readability only */
#endif
#define DXB_EPERM	-1      /* Operation not permitted */
#define DXB_ENOMEM	-12	/* Out of memory */
#define DXB_EACCES	-13	/* Permission denied */
#define DXB_EINVAL	-22	/* Invalid argument */
#define DXB_EMCHAN	-24	/* Too many open channels */
#define DXB_ENOSPACE	-100	/* No space in queue */
#define DXB_ENOCRED	-101	/* Insufficient credits */
#define DXB_ENOCB	-102	/* Need a callback routine */

/*
 *	Priority values for readability only (highpri, where used, is boolean)
 */
#define	DXB_PRI_NORMAL	0	/* Normal priority */
#define	DXB_PRI_HIGH	1	/* High priority */

/*
 *	Flag in the channel number of a callback which indicates this
 *	is not the 1st callback, for the dxbuf option.
 */
#define	DXB_DXBUF_CB	0x8000
#define	DXB_CHAN_MASK	0x7fff

/*
 *	For debugging purposes, a few global variables are exposed
 */
extern DXB	*LibdxbDxb;		/* Addr of last dxb instance initted */
extern DXBCHAN	*LibdxbChan;		/* last dxb->chan location */
extern int	LibdxbChanOffset;	/* Offset of dxb->chan from dxb */

/************************************************************************/
/*									*/
/*	API Abstraction level functions and macros provided by the	*/
/*	library user (the porting layer).				*/
/*									*/
/************************************************************************/

/*
 *	dxb_set_userdata:
 *	dxb_get_userdata:
 *	dxb_set_syncdata:
 *	dxb_get_syncdata:
 *
 *	set/get private user data
 *
 *	These are provided *by the library* to allow the user to set/get
 *	the private per-DXB-instance user data.  The user data is an
 *	arbitrary value that can be used within the porting layer
 *	macros below for whatever purpose the library user might
 *	require for a specific host OS.  The data itself can be
 *	anything that will fit into a DXBARG.
 *
 *	The set function are done as (potential) inline functions
 *	to allow you to set a breakpoint if desired.  Otherwise,
 *	they are so simple that they could have been macros.
 */
DXB_INLINE(
void
dxb_set_userdata(DXB *dxb, DXB_ARG userdata),
{
	dxb->userdata = userdata;
}
)

#define	dxb_get_userdata(DXB) \
		(DXB->userdata)

DXB_INLINE(
void
dxb_set_syncdata(DXB *dxb, DXB_ARG syncdata),
{
	dxb->syncdata = syncdata;
}
)

#define	dxb_get_syncdata(DXB) \
		(DXB->syncdata)

/*
 *	The following user-supplied functions could also be macros defined
 *	in your dxbtypes.h file.  If they are defined as macros, note the
 *	type of the arguments and return value.
 */

/*
 *	dxb_malloc:
 *
 *	User-supplied routine or macro that allocates "kernel" memory
 *	with no particular alignment.
 */
#ifndef dxb_malloc	/* Could be a macro */
	extern void	*dxb_malloc(unsigned int size);
#endif

/*
 *	dxb_free:
 *
 *	User-supplied routine or macro that frees memory allocated by
 *	dxb_malloc().
 */
#ifndef dxb_free	/* Could be a macro */
	extern void	dxb_free(void *buf);
#endif

/*
 *	dxb_phys_page_size:
 *
 *	User-supplied routine or macro that returns the physical page
 *	size of the operating system.  Typically 4096.
 */
#ifndef dxb_phys_page_size	/* Could be a macro */
	extern int	dxb_phys_page_size(void);
#endif

/*
 *	dxb_alloc_page:
 *
 *	User-supplied routine or macro that allocates a single page of
 *	kernel memory aligned on a page boundary.
 */
#ifndef dxb_alloc_page	/* Could be a macro */
	extern void	*dxb_alloc_page(void);
#endif

/*
 *	dxb_free_page:
 *
 *	User-supplied routine or macro that frees memory allocated by
 *	dxb_alloc_page().
 */
#ifndef dxb_free_page	/* Could be a macro */
	extern void	dxb_free_page(void *page);
#endif

/*
 *	dxb_virt2bus:
 *
 *	User-supplied routine or macro that converts a kernel virtual
 *	address into a bus address (sometimes called physical address).
 */
#ifndef dxb_virt2bus	/* Could be a macro */
	extern uint32	dxb_virt2bus(void *virt);
#endif

/*
 *	dxb_syncwrite:
 *	dxb_syncread:
 *
 *	User-supplied macros to sync (flush or invalidate host cache)
 *	when writing or reading from any DXB data which is DMA'ed
 *	to/from the board.  The DXB library will call these to insure
 *	that any data it is reading/writing to these areas is sync'ed.
 *
 *	The arguments to this function/macro are:
 *	"ptr"		is a host virtual address of the area to be sync'ed.
 *	"len"		is the length of the area to be sync'ed in bytes.
 *	"syncdata"	is an arbitrary value that the user has set with
 *			the dxb_set_syncdata() initialization routine. This
 *			might be a bus number or some such thing.
 *
 *	These are needed, for example, on a Sparc, where they could
 *	be defined as something like this:
 *
 *		#define dxb_syncwrite(PTR, LEN, SYNCDATA) \
 *			mmu_sync_dma(PTR, LEN, (struct linux_sbus *) SYNCDATA)
 *
 *	At DXB init time, a Sparc user would make a call like this:
 *
 *		dxb_set_syncdata(dxb, (DXB_ARG) mystruct->sbus_info);
 */
#undef  DXB_USE_SYNCWRITE
#ifndef dxb_syncwrite	/* Must be a macro */
	#define dxb_syncwrite(X, Y, Z)
#else
	#define DXB_USE_SYNCWRITE
#endif

#undef  DXB_USE_SYNCREAD 
#ifndef dxb_syncread	/* Must be a macro */
	#define dxb_syncread(X, Y, Z)
#else
	#define DXB_USE_SYNCREAD
#endif

/*
 *	dxb_printf:
 *
 *	User-supplied routine or macro that prints debugging messages.
 */
#ifndef dxb_printf	/* Could be a macro */
	extern void	dxb_printf(const char *fmt, ...);
#endif

/*
 *	dxb_assert:
 *
 *	User-supplied assert macro that checks assertion and causes
 *	fatal exit if code has been compiled "for debugging".
 */
#ifndef dxb_assert	/* Must be a macro */
	#define dxb_assert(X)
#endif

/*
 *	dxb_insist:
 *
 *	User-supplied assert macro that checks assertion and causes
 *	fatal exit regardless of whether or not code is compiled for
 *	debugging.
 */
#ifndef dxb_insist	/* Must be a macro */
	#define dxb_insist(X)
#endif

/*
 *	dxbtypes.h:
 *
 *	The user needs to supply a header file with the scalar data types
 *	used by this library (uint8/uint16/uint32, etc.).  These types
 *	may be defined by simply #including some other system header,
 *	or you may have to typedef them yourself, or you may have to
 *	do a mixture of both.  You can also use this file to define
 *	macros instead of true functions for the OS abstraction layer.
 *
 *	The following files can be used as a guide for what you need:
 *		dense/houston/drv/linux/dxbtypes.h
 *		dense/houston/drv/nw/dxbtypes.h
 *		dense/houston/drv/uw7/dxbtypes.h
 *		dense/houston/fw/hou2fep/dxbtypes.h
 *
 *	Also, if your compiler can do GCC style inline functions, you can
 *		#define DXB_HAS_INLINE 1
 *	to take advantage of this.
 */



/************************************************************************/
/*
 *	Rick doesn't like to read or type long names
 */
/************************************************************************/
#define	h2a	host2adapter_q
#define	a2h	adapter2host_q
#define	h2a_in	host2adapter_in
#define	h2a_out	host2adapter_out
#define	a2h_in	adapter2host_in
#define	a2h_out	adapter2host_out
