
/*
 Copyright 1999-2000 by Digi International (www.digi.com)

 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2, or (at your option)
 any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY, EXPRESS OR IMPLIED; without even the 
 implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR 
 PURPOSE.  See the GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

/*#define MAKE_TEST_PROGRAM */

#include <stdlib.h>
#include <stdarg.h>
#include <string.h>
#include <stdio.h>
#include <ctype.h>
#include <errno.h>
#include "tracelog.h"
/* #include "isdnshow.h" */

/*****************************************************************************
** The following macros (SWAP16 and SWAP32) implement byte swapping and
** byte alignment for machines that are either not little endian or which
** have alignment restrictions on memory accesses larger than one character
** (currently the Sun SPARC architecture).  Accesses to structure elements
** larger than one byte (UINT16, UINT32, etc.) should use these.
******************************************************************************/
#undef BYTE
typedef unsigned char BYTE ;

#ifdef sparc
#if 0
#define SWAP16(x) (((BYTE *)&(x))[0] + (((BYTE *)&(x))[1] * 256))
#define SWAP32(x) (((BYTE *)&(x))[0] + (((BYTE *)&(x))[1] << 8) + (((BYTE *)&(x))[2] << 16) + (((BYTE *)&(x))[3] << 24))
#else
#define SWAP32(l) (\
(((l)&0xFF000000)>>24) | \
(((l)&0x00FF0000)>> 8) | \
(((l)&0x0000FF00)<< 8) | \
(((l)&0x000000FF)<<24) )
#define SWAP16(s) ((((s) >> 8) & 0xFF) | (((s) << 8) & 0xFF00))
#endif
#else
#define SWAP16(x) (x)
#define SWAP32(x) (x)
#endif
#define NOSWAP16(x) (x)
#define NOSWAP32(x) (x)

static int ascii_trace(BYTE *data, int data_len);
static int dch_trace_lapd(TLF_DCH_RECORD *te, int flags);
static int dch_trace_il3_msg(BYTE *data);
static int dch_trace_il3_info(BYTE *data, int sz, int offset, int flags);
static int hdump(BYTE *data, int datalen);

#define ERR_FRAME_FILTERED 200
#define MSD_MAX 54 /* max length of Module Specific Area part of trace line */
#define EHM_MAX 45  /* max length of element highlight mnemonic data area */

#define TLF_DATA(ptr) ((BYTE *)((ptr)+1)) /* ptr is of type (TLF_HEADER *) */
#define TLF_DCH_END_LEN(ptr, dlen) ((TLF_LENGTH *) (((char *)(ptr)) + sizeof(TLF_DCH_RECORD) + (dlen)))
#define MIN(a, b) ((a) < (b) ? (a) : (b))

static char *id_outbuf;
static long  id_outbuf_sz;

/* print to buffer, with check for passing end of buffer.  Maximum output */
/* length is 100 chars, 100 chars worth of stack is needed... */
static int iprintf(const char *format, ... )
{
    int        rtn;
    va_list args;
    char    buf[100];

    va_start(args, format);
    rtn = vsprintf(buf, format, args);
    if (rtn >= id_outbuf_sz)
        return ERR_NO_MEM;
    strcpy(id_outbuf, buf);
    id_outbuf += rtn;
    id_outbuf_sz -= rtn;
    return OK;
}


int isdn_dump(TLF_HEADER *te, char *out_buf, long out_buf_size, int flags)
{
    int                 rtn;
    char               *s;
    BYTE               *data;
    int                 do_hex_dump = NO;
    int                 data_len;
    TLF_ASCII_RECORD   *tlf_a = (TLF_ASCII_RECORD *) te;
    TLF_HEX_RECORD     *tlf_h = (TLF_HEX_RECORD *) te;
    TLF_DCH_RECORD     *tlf_d = (TLF_DCH_RECORD *) te;


    id_outbuf = out_buf;
    id_outbuf_sz = out_buf_size;
    *id_outbuf = '\0';

    
    /* write the header-Banner line */
    if (flags & IDF_MSG_HEADER)
        {
        switch (NOSWAP16(te->TypeId))
            {
            case TLF_TYPE_ASCII:
                    s = "          MESSAGE";
                    break;
            case TLF_TYPE_DCH:
                    s = "S  ADDRESS  FTYP P NS  NR  CRV    MESSAGE";
                    break;
            case TLF_TYPE_HEX:
            case TLF_TYPE_BCH:
            case TLF_TYPE_PPP:
            case TLF_TYPE_X25:
            default:
                    s = "";
                    break;
            }
        rtn = iprintf("SEQ TIME        B LN  MOD %s\n", s);
        if (rtn != OK)
            goto err_exit;
        }

    /* write the module independent parts of the trace line */
    rtn = iprintf("%03d %02d:%02d:%02d.%02d %01d %03d ",
                NOSWAP16(te->SeqNum) % 1000,
                te->TimeStamp.Hour,
                te->TimeStamp.Minute,
                te->TimeStamp.Second,
                te->TimeStamp.CentiSecond,
                te->DeviceId,
                te->LineId);
    if (rtn != OK)
        goto err_exit;

    /* write the module dependent parts of the trace line. */
    /* Any additional lines necessary are decided on here also... */
    if (flags & IDF_HEX_DUMP)
        do_hex_dump = YES;
    switch (NOSWAP16(te->TypeId))
        {
        case TLF_TYPE_ASCII:
            if (flags & IDF_NO_ASCII_HEX_DUMP)
                do_hex_dump = NO;
            data = TLF_DATA(tlf_a);
            data_len = NOSWAP32(tlf_a->DataLen);
            /*rtn = iprintf("ASC %.*s\n", MIN(MSD_MAX, data_len), data); */
            rtn = ascii_trace(data, data_len);
            if (rtn != OK)
                goto err_exit;
            break;
        case TLF_TYPE_HEX:
            data = TLF_DATA(tlf_h);
            data_len = NOSWAP32(tlf_h->DataLen);
            do_hex_dump = YES;
            rtn = iprintf("HEX\n");
            if (rtn != OK)
                goto err_exit;
            break;
        case TLF_TYPE_DCH:
            do_hex_dump = NO;   /* D-Channel hex dumps done in dch_trace_lapd */
            rtn = dch_trace_lapd(tlf_d, flags);
            if (rtn == ERR_FRAME_FILTERED)
                {
                out_buf[0] = '\0';
                return OK;
                }
            if (rtn != OK)
                goto err_exit;
            break;
        case TLF_TYPE_BCH:
            /* B-Channel structure not defined yet, so do hex dump of frame */
            data = (BYTE *) te + sizeof(TLF_HEADER);
            data_len = NOSWAP32(te->Len) -
                              (sizeof(TLF_HEADER) + sizeof(TLF_LENGTH));
            do_hex_dump = YES;
            rtn = iprintf("BCH\n");
            if (rtn != OK)
                goto err_exit;
            break;
        case TLF_TYPE_PPP:
            /* PPP structure not defined yet, so do hex dump of frame */
            data = (BYTE *) te + sizeof(TLF_HEADER);
            data_len = NOSWAP32(te->Len)-(sizeof(TLF_HEADER)+sizeof(TLF_LENGTH));
            do_hex_dump = YES;
            rtn = iprintf("PPP\n");
            if (rtn != OK)
                goto err_exit;
            break;
        case TLF_TYPE_X25:
            /* X.25 structure not defined yet, so do hex dump of frame */
            data = (BYTE *) te + sizeof(TLF_HEADER);
            data_len = NOSWAP32(te->Len)-(sizeof(TLF_HEADER)+sizeof(TLF_LENGTH));
            do_hex_dump = YES;
            rtn = iprintf("X25\n");
            if (rtn != OK)
                goto err_exit;
            break;
        default:
            /* undefined frame type; assume bad input data (no hex dump) */
            do_hex_dump = NO;
            rtn = iprintf("UNKNOWN FRAME TYPE %d\n", NOSWAP16(te->TypeId));
            if (rtn != OK)
                goto err_exit;
            break;
        }

    if (do_hex_dump == YES)
        {
        rtn = hdump(data, data_len);
        if (rtn != OK)
            return rtn;
        }

    if (flags & IDF_WHITESPACE)
        {
        rtn = iprintf("\n");
        if (rtn != OK)
            return rtn;
        }

    return OK;

err_exit:
    return rtn;
}

/*****************************************************************************
** This routine deals with arbitrary contents for the ASCII trace log record 
** type as follows: 
** 1) expands tabs (tab stops every 8th column) 
** 2) ends the current line at 80 columns, or on finding a newline 
** 3) replacing all other control chars with periods (.) 
** 4) lines after the first are prefixed with ' + ' 
** extra credit feature (may not be done): wrap line at last whitespace 
** instead of at column 80.
****************************************************************************/
static int ascii_trace(BYTE *data, int data_len)
{
#define MOD_START_COL 26
    int rtn = OK;
    int col = MOD_START_COL;  /* zero based... */
    BYTE *d = data;
    char buf[89]; /* 89 = buffer len + tab expansion + null char... */
    char *bp = buf;
   
    rtn = iprintf("ASC ");
    if (rtn != OK)
        return rtn;

    if (data_len == 0 || (data_len == 1 && data[data_len] == '\0'))
        {
        rtn = iprintf("\n");
        return rtn;
        }
    if (data[data_len - 1] == '\0')
        data_len--;

    while (d < data + data_len)
        {
        if (*d == '\t')
            {
            do
                {
                *bp++ = ' ';
                }
            while (++col % 8);
            }
        else if (*d == '\n')
            {
            *bp++ = '\n';
            *bp++ = '\0';
            rtn = iprintf("%s", buf);
            if (rtn != OK)
                return rtn;
            strcpy(buf, " + ");
            col = strlen(buf);
            bp = buf + col;
            }
        else if (*d < ' ' || *d > '~') /* character is not visible... */
            {
            *bp++ = '.';
            col++;
            }
        else  /* visible character */
            {
            *bp++ = *d;
            col++;
            }
        d++;
        if (col >= 80 || d == data + data_len)
            {
            buf[80] = '\n';
            buf[81] = '\0';
            *bp++ = '\n';
            *bp++ = '\0';
            rtn = iprintf("%s", buf);
            if (rtn != OK)
                return rtn;
            strcpy(buf, " + ");
            col = strlen(buf);
            bp = buf + col;
            }
        }
    return OK; 
}

static int dch_trace_lapd(TLF_DCH_RECORD *te, int flags)
{
    typedef struct {
        BYTE    addr1;
        BYTE    addr2;
        BYTE    ctrl1;
        BYTE    ctrl2;
        BYTE    info[1];
    } lapd_header;
#define REC(te) (((te)->Flags & DCH_FLAG_DIR_RX) == DCH_FLAG_DIR_RX)
#define IS_NT(te)  (((te)->Flags & DCH_FLAG_NT_SIDE) == DCH_FLAG_NT_SIDE)
#define IS_SOURCE(te) (!REC(te) ^ IS_NT(te))
#define SAPI(h) ((BYTE)((h)->addr1 & 0xFC) >> 2)
#define TEI(h)  ((BYTE)((h)->addr2 & 0xFE) >> 1)
#define C_R(h) ((BYTE)((h)->addr1 & 0x02) >> 1)
#define IS_CMD(h, te) (C_R(h) ^ IS_SOURCE(te))

#define IS_I(h) (((h)->ctrl1 & 0x01) == 0)
#define IS_S(h) (((h)->ctrl1 & 0x03) == 0x01)
#define IS_U(h) (((h)->ctrl1 & 0x03) == 0x03)
#define IS_UI(h) (IS_U((lh)) && (MFB((lh)) == 0))

#define SFB(h) ((BYTE)((h)->ctrl1 & 0x0C) >> 2)
#define MFB(h) ((BYTE)((h)->ctrl1 & 0xEC) >> 2)
#define NS(h)  ((BYTE)((h)->ctrl1 & 0xFE) >> 1)
#define NR(h)  ((BYTE)((h)->ctrl2 & 0xFE) >> 1)
#define PF_BIT(h) (IS_U(h) ? ((BYTE)((h)->ctrl1&0x10)>>4) : ((h)->ctrl2&0x01))
/*#define PF(h, te) (!PF_BIT(h) ? '-' : (IS_CMD(h, te) ? 'P' : 'F')) */
#define PF(h, te) (PF_BIT(h) ? '1' : '0')

    lapd_header *lh = (lapd_header *) TLF_DATA(te);
    int rtn = OK;
    char *ftyp;
    char *msg;
    int has_info_elements = NO;

    rtn = iprintf("DCH %s %.2d-%.3d-%c ",
#if 0
        IS_SOURCE(te) ? "TE" : "NT",
#else
	REC(te) ? "RX" : "TX",
#endif
        SAPI(lh), TEI(lh), 
#if 0
        IS_CMD(lh, te) ? 'C' : 'R'); 
#else
        C_R(lh) ? 'C' : 'R'); 
#endif
    if (rtn != OK)
        return rtn;

    if (IS_I(lh))
        {
        ftyp = "I";

        rtn = iprintf("%-4s %c %.3d %.3d ",
             ftyp, PF(lh, te), NS(lh), NR(lh));
        }
    else if (IS_S(lh))
        {
        switch (SFB(lh))
            {
            case 0:  ftyp = "RR";  break;
            case 1:  ftyp = "RNR"; break;
            case 2:  ftyp = "REJ"; break;
            default: ftyp = "";    break;
            }
        rtn = iprintf("%-4s %c     %.3d ",
             ftyp, PF(lh, te), NR(lh));
        }
    else if (IS_U(lh))
        {
        switch (MFB(lh))
            {
            case 0x00:  ftyp = "UI";    break;
            case 0x03:  ftyp = "DM";    break;
            case 0x10:  ftyp = "DISC";  break;
            case 0x18:  ftyp = "UA";    break;
            case 0x1B:  ftyp = "SABE";  break;
            case 0x21:  ftyp = "FRMR";  break;
            case 0x2B:  ftyp = "XID";   break;
            default: ftyp = "";         break;
            }
        rtn = iprintf("%-4s %c         ",
             ftyp, PF(lh, te));
        }

    if (rtn != OK)
        return rtn;
    
    /****** process information frame of messages that have one. */
#define MEI(h) ((h)->ctrl2)
#define REF(h) ((((h)->info[0]) << 8) + ((h)->info[1]))
#define MSGTYPE(h) ((h)->info[2])
#define AI(h)  ((BYTE)((h)->info[3] & 0xFE) >> 1)
    /* look for TEI management frames */
    if ((SAPI(lh) == 63) && (TEI(lh) == 127) && IS_UI(lh) && (MEI(lh) == 0x0F))
        {
        /* if IDF_FILTER_L2 is set, this message is filtered out... */
        if (flags & IDF_FILTER_L2)
            return ERR_FRAME_FILTERED;
        /* This is a TEI Management message */
        switch (MSGTYPE(lh)) {
            case 1: msg = "REQ"; break;
            case 2: msg = "ASN"; break;
            case 3: msg = "DEN"; break;
            case 4: msg = "CRQ"; break;
            case 5: msg = "CRS"; break;
            case 6: msg = "REM"; break;
            case 7: msg = "VER"; break;
            default: msg = NULL; break;
            }
        if (msg == NULL)
            rtn = iprintf("\n");
        else
            rtn = iprintf("       ID_%s R:%.5u A:%.3d\n", msg, REF(lh), AI(lh));
        }
    else if (IS_I(lh))
        {
        /* assume any I frames left at this point are level 3 messages */
        has_info_elements = YES;
        rtn = dch_trace_il3_msg(TLF_DATA(te) + 4);
        }
    else if (IS_UI(lh))
        {
        /* assume any UI frames left at this point are level 3 messages */
        has_info_elements = YES;
        rtn = dch_trace_il3_msg(TLF_DATA(te) + 3);
        }
    else
        {
        if (flags & IDF_FILTER_L2)
            return ERR_FRAME_FILTERED;
        rtn = iprintf("\n");
        }
    if (rtn != OK)
        return rtn;
    
    if (flags & IDF_HEX_DUMP)
        {
        rtn = hdump(TLF_DATA(te), NOSWAP32(te->DataLen));
        if (rtn != OK)
            return rtn;
        }

    if (has_info_elements == YES)
        {
        rtn = dch_trace_il3_info(TLF_DATA(te), NOSWAP32(te->DataLen),
            3+IS_I(lh), flags);
        if (rtn != OK)
            return rtn;
        }
    return OK;
}



static int dch_trace_il3_msg(BYTE *data)
{
    int rtn;
    int i;
    int pd;
    long crv = 0;
    int crv_flag = 0;
    unsigned msgtype;
    char *msgstr;
    char buf[60];

    /* check for a known protocol discriminator */
    pd = *data++;
    if (pd != 0x08 && pd != 0x41 && pd != 0x03) /* CBP 10/29 */
        {
        rtn = iprintf(" ****UNKNOWN PROTOCOL (%02XH)\n", pd);
        return rtn;
        }

    /* get the call reference value */
    i = *data++ & 0x0F;
    if (i > 0)
        {
        crv_flag = (BYTE)(*data & 0x80) >> 7;
        crv = *data++ & 0x7F;
        for ( ; i > 1; i--)
            crv = (crv * 256) + *data++;
        }

    /* Decode the message type */
    msgtype = *data++ & 0x7F;
    switch (pd) {
    case 0x03: /* handle PRI maintenance messages */
    	switch ((BYTE)msgtype)
        {
        case 0x07: msgstr = "SERVICE ACKNOWLEDGE";  break;
        case 0x0F: msgstr = "SERVICE";  break;
        default:   sprintf(buf, "Reserved L3 msg %02X", msgtype);
                   msgstr = buf;
                   break;
        }
	break;
    default:
    	switch ((BYTE)msgtype)
        {
        case 0x00: sprintf(buf, "NATIONAL MSG %02X", *data);
                   msgstr = buf;
                   break;
        case 0x01: msgstr = "ALERTING";             break;
        case 0x02: msgstr = "CALL PROCEEDING";      break;
        case 0x07: msgstr = "CONNECT";              break;
        case 0x0F: msgstr = "CONNECT ACKNOWLEDGE";  break;
        case 0x03: msgstr = "PROGRESS";             break;
        case 0x05: msgstr = "SETUP";                break;
        case 0x0D: msgstr = "SETUP ACKNOWLEDGE";    break;
        case 0x26: msgstr = "RESUME";               break;
        case 0x2E: msgstr = "RESUME ACKNOWLEDGE";   break;
        case 0x22: msgstr = "RESUME REJECT";        break;
        case 0x25: msgstr = "SUSPEND";              break;
        case 0x2D: msgstr = "SUSPEND ACKNOWLEDGE";  break;
        case 0x21: msgstr = "SUSPEND REJECT";       break;
        case 0x20: msgstr = "USER INFORMATION";     break;
        case 0x45: msgstr = "DISCONNECT";           break;
        case 0x4D: msgstr = "RELEASE";              break;
        case 0x5A: msgstr = "RELEASE COMPLETE";     break;
        case 0x46: msgstr = "RESTART";              break;
        case 0x4E: msgstr = "RESTART ACKNOWLEDGE";  break;
        case 0x60: msgstr = "SEGMENT";              break;
        case 0x79: msgstr = "CONGESTION CONTROL";   break;
        case 0x7B: msgstr = "INFORMATION";          break;
        case 0x6E: msgstr = "NOTIFY";               break;
        case 0x7D: msgstr = "STATUS";               break;
        case 0x75: msgstr = "STATUS ENQUIRY";       break;
        default:   sprintf(buf, "Reserved L3 msg %02X", msgtype);
                   msgstr = buf;
                   break;
        }
	break;
    }


    /* write the Call Reference Value and Message Type string */
    rtn = iprintf("%c%.5ld %s\n", (crv_flag == 0) ? 'O' : 'D',
            crv % 100000, msgstr);
    if (rtn != OK)
        return rtn;

    return OK;
}


static int dch_trace_il3_info(BYTE *data, int sz, int offset, int flags)
{
    int rtn;
    int i;
    BYTE *eod = data + sz;
    char *msgstr;
    char *esinfo;
    char buf[60];
    int active_codeset = 0;
    int shifted_codeset = 0;
    BYTE ie;
    int ie_len;
    int pd;
    char *s = (char *) data;
    char *t;

    /* see if information elements are to be displayed */
    if (!(flags & IDF_INFO_ELEMENTS)) {
        return OK;
    }

    /* advance to start of level 3 packet */
    data += offset;

    /* check for a known protocol discriminator */
    pd = *data;
    if (pd != 0x08 && pd != 0x41 && pd != 0x03) 
        return OK;

    /* advance to start of information element section */
    data++;
    offset += (*data & 0x0F) + 3;
    data += (*data & 0x0F) + 1;
    if (*data++ == 0)  /* Nationally specific msg escape (msg in next byte) */
        {
        data++;
        offset++;
        }

    /* Print Information element header line if any present */
    if (data < eod)
        {
        if (flags & IDF_INFO_HEADER)
            {
            rtn = iprintf("        indx #  ln element name       "
                            "element highlight mnemonic\n");
            if (rtn != OK)
                return rtn;
            }
        }

    /* Process any Information Elements present */
    while (data < eod)
        {
        ie = *data;
        buf[0] = '\0';

        /* handle single byte Information Elements */
        if ((ie & 0x80) && (shifted_codeset == 0))
            {
            switch (ie & 0xF0)
                {
                case 0x90:
                        if (ie & 0x08)
                            {
                            msgstr = "NON-LOCK SHIFT";
                            shifted_codeset = ie & 0x07; /* non-locking shift */
                            }
                        else
                            {
                            /* locking shift */
                            msgstr = "LOCKING SHIFT";
                            if (active_codeset < (int)(ie & 0x07))
                                active_codeset = ie & 0x07;
                            shifted_codeset = active_codeset;
                            }
                        sprintf(buf, "TO CODESET %d", shifted_codeset);
                        break;
                case 0xA0: if (ie == 0xA0)
                            msgstr = "MORE DATA";
                        else
                            msgstr = "SENDING COMPLET";
                        break;
                case 0xB0: msgstr = "CONGESTION LVL";
                        if (ie == 0xB0)
                            strcpy(buf, "RECEIVER READY");
                        else if (ie == 0xBF)
                            strcpy(buf, "RECEIVER NOT READY");
                        else
                            sprintf(buf, "UNDEFINED [%02X]", ie & 0x0F);
                        break;
                case 0xD0: msgstr = "REPEAT INDICATO";
                        break;
                default:   msgstr = "RESERVED";
                }
            rtn = iprintf("        %04X %02X    %-15s",
                        offset, ie, msgstr);
            if (rtn != OK)
                return rtn;
            if (strlen(buf))
                rtn = iprintf(" : %s\n", buf);
            else
                rtn = iprintf("\n");
            if (rtn != OK)
                return rtn;
            data++;
            offset++;
            continue;
            }
        else if (*data & 0x80)
            {
            /* codeset is not than 0: put out an "I don't know" msg */
            rtn = iprintf("        %04X %02X                    : UNKNOWN CODESET %d\n",
                         offset, ie, shifted_codeset);
            if (rtn != OK)
                return rtn;
            shifted_codeset = active_codeset;
            offset++;
            data++;
            }
        else if (shifted_codeset == 0)
            {
            /* handle multiple byte Information Elements */
            ie_len = data[1];
            esinfo = "";
            buf[0] = '\0';
            switch (ie)
                {
                case 0x01:  /* Change Status (pd = 0x3 = Maintenance) */
			if (pd != 0x3) {
                        	sprintf(buf, "UNDEFINED [%02X]", ie);
                        	msgstr = buf;
                        	break;
			}
                        msgstr = "CHANGE STATUS";
                        if ((data[2] & 0xf8) == 0xc0)
                            s = "NEW STATUS";
                        else
                            s = "BAD FORMAT";
                        strcpy(buf, s);
                        switch (data[2] & 0x07) {
				case 0x00: strcat(buf, ", IN SERVICE");
					break;
				case 0x01: strcat(buf, ", MAINTENANCE");
					break;
				case 0x02: strcat(buf, ", OUT OF SERVICE");
					break;
				default: strcat(buf, ", BAD STATE");
					break;
			}
                        buf[EHM_MAX] = '\0';
                        esinfo = buf;
                        break;
                case 0x04:  /* Bearer Capability */
                        {
                        BYTE *d;
                        msgstr = "BEARER CAPABIL";
                        #define XCAP(x) ((x)[2] & 0x1F)
                        if ((data[2] & 0x60) != 0)  /* CCITT coding std? */
                            {
                            sprintf(buf, "Unknown coding standard [%d]", 
                                    (BYTE)(data[2] & 0x60) >> 5);
                            esinfo = buf;
                            break;
                            }
                        switch (XCAP(data)) /*information transfer cap */
                            {
                            case 0x00: s = "SPEECH";                    break;
                            case 0x08: s = "UNRESTRICTED DIGITAL";      break;
                            case 0x09: s = "RESTRICTED DIGITAL";        break;
                            case 0x10: s = "3.1KHZ AUDIO";              break;
                            case 0x11: s = "UNRESTR DIGITAL W/TONES";   break;
                            case 0x18: s = "VIDEO";                     break;
                            default:   s = "UNDEFINED [%02X]";          break;
                            }
                        /* add transfer rate where appropriate */
                        t = "";
                        d = &data[4];
                        if ((XCAP(data) == 0x08) /* unrestricted digital */
                          ||(XCAP(data) == 0x09) /* restricted digital */
                          ||(XCAP(data) == 0x11) /* unres dig w/tones/announce*/
                          ||(XCAP(data) == 0x18))/* video */
                            {
                            switch (data[3] & 0x7F)
                                {
                                case 0x10: t = " 64K";            break;
                                case 0x11: t = " 2x64K";          break;
                                case 0x13: t = " 384K";           break;
                                case 0x15: t = " 1536K";          break;
                                case 0x17: t = " 1920K";          break;
                                case 0x18: t = " MULTIRATE"; d++; break;
                                case 0x40: t = " PACKET";         break;
                                default:   t = "";                break;
                                }   
                            }
                        if (((XCAP(data) == 0x08) && ((*d == 0x21) ||
                            (*d == 0x28))) || ((XCAP(data) == 0x10)
                             && (*d  == 0x22)))
                            /* User info layer1, rate adaption */
                            {
                            switch (*++d & 0x1F) 
                                {
                                case 0x00: t = " I.460 RA";       break;
                                case 0x01: t = " 0.6K RA";        break;
                                case 0x02: t = " 1.2K RA";        break;
                                case 0x03: t = " 2.4K RA";        break;
                                case 0x04: t = " 3.6K RA";        break;
                                case 0x05: t = " 4.8K RA";        break;
                                case 0x06: t = " 7.2K RA";        break;
                                case 0x07: t = " 8K RA";          break;
                                case 0x08: t = " 9.6K RA";        break;
                                case 0x09: t = " 14.4K RA";       break;
                                case 0x0A: t = " 16K RA";         break;
                                case 0x0B: t = " 19.2K RA";       break;
                                case 0x0C: t = " 32K RA";         break;
                                case 0x0E: t = " 48K RA";         break;
                                case 0x0F: t = " 56K RA";         break;
                                case 0x15: t = " 0.1345K RA";     break;
                                case 0x16: t = " 0.100K RA";      break;
                                case 0x17: t = " 0.075K/1.2K RA"; break;
                                case 0x18: t = " 1.2K/0.075K RA"; break;
                                case 0x19: t = " 0.05K RA";       break;
                                case 0x1A: t = " 0.075K RA";      break;
                                case 0x1B: t = " 0.110K RA";      break;
                                case 0x1C: t = " 0.150K RA";      break;
                                case 0x1D: t = " 0.200K RA";      break;
                                case 0x1E: t = " 0.300K RA";      break;
                                case 0x1F: t = " 12K RA";         break;
                                default: t = " RESERVED RA";      break;
                                }
                            }
                        sprintf(buf, s, XCAP(data));
                        strcat(buf, t);
                        esinfo = buf;
                        break;
                        }
                case 0x10:  /* Call Identity */
                        msgstr = "CALL IDENTITY";
                        break;
                case 0x14:  /* Call State */
                        msgstr = "CALL STATE";
                        switch (data[2] & 0x3F)
                            {
                            case 0x00: s = "NULL";                     break;
                            case 0x01: s = "CALL INITIATED";           break;
                            case 0x02: s = "OVERLAP SENDING";          break;
                            case 0x03: s = "OUTGOING CALL PROCEEDING"; break;
                            case 0x04: s = "CALL DELIVERED";           break;
                            case 0x06: s = "CALL PRESENT";             break;
                            case 0x07: s = "CALL RECEIVED";            break;
                            case 0x08: s = "CONNECT REQUEST";          break;
                            case 0x09: s = "INCOMING CALL PROCEEDING"; break;
                            case 0x0A: s = "ACTIVE";                   break;
                            case 0x0B: s = "DISCONNECT REQUEST";       break;
                            case 0x0C: s = "DISCONNECT INDICATION";    break;
                            case 0x0F: s = "SUSPEND REQUEST";          break;
                            case 0x11: s = "RESUME REQUEST";           break;
                            case 0x13: s = "RELEASE REQUEST";          break;
                            case 0x16: s = "CALL ABORT";               break;
                            case 0x19: s = "OVERLAP RECEIVING";        break;
                            default: s = "UNDEFINED"; break;
                            }
                        t = buf + sprintf(buf, "%s", s);
                        if (strcmp(s, "UNDEFINED") == 0)
                            sprintf(t, " [%02X]", data[2] & 0x3F);
                        esinfo = buf;
                        break;
                case 0x6C:  /* Calling party number */
                        msgstr = "CALLING NUMBER";
                        t = (char *)data + 2;
                        i = ie_len - 1;
                        if ((*t++ & 0x80) == 0)   /* skip octet 3a if present */
                            {
                            t++;
                            i--;
                            }
                        sprintf(buf, "%.*s", MIN(i, EHM_MAX), t);
                        esinfo = buf;
                        break;
                case 0x70:  /* Called party number */
                case 0x71:  /* Called party subaddress */
                case 0x6D:  /* Calling party subaddress */
                        if (ie == 0x70)
                            msgstr = "CALLED NUMBER";
                        else if (ie == 0x71)
                            msgstr = "CALLED SUBADDR";
                        else
                            msgstr = "CALLING SUBADDR";
                        /* this displays the subaddress (if in ascii...) */
                        sprintf(buf, "%.*s", MIN(ie_len-1, EHM_MAX), data + 3);
                        esinfo = buf;
                        break;
                case 0x08:  /* Cause */
                        msgstr = "CAUSE";
                        t = (char *) &(data[2]);
                        /* 1TR6 has an ie_len of 1, cause code is in 1st byte */
                        if (ie_len > 1)  /* CBP 10/29 for 1TR6 */
                            {
                            /* q.850-style cause code */
                            if ((*t++ & 0x80) == 0)
                                t++;  /* skip recommendation field, if present*/
                            }
                        switch (*t & 0x7F)
                            {
                            case 0x01: s = "UNALLOCATED NUMBER";         break;
                            case 0x02: s = "NO ROUTE TO NETWORK";        break;
                            case 0x03: s = "NO ROUTE TO DESTINATION";    break;
                            case 0x06: s = "CHANNEL UNACCEPTABLE";       break;
                            case 0x07: s = "CALL AWARDED";               break;
                            case 0x10: s = "NORMAL CALL CLEARING";       break;
                            case 0x11: s = "USER BUSY";                  break;
                            case 0x12: s = "NO USER RESPONDING";         break;
                            case 0x13: s = "NO ANSWER FROM USER";        break;
                            case 0x15: s = "CALL REJECTED";              break;
                            case 0x16: s = "NUMBER CHANGED";             break;
                            case 0x1A: s = "NON-SELECT USER CLR";        break;
                            case 0x1B: s = "DESTINATION OUT OF ORDER";   break;
                            case 0x1C: s = "INVALID NUMBER FORMAT";      break;
                            case 0x1D: s = "FACILITY REJECTED";          break;
                            case 0x1E: s = "RESPONSE TO STATUS ENQUIRY"; break;
                            case 0x1F: s = "NORMAL UNSPECIFIED";         break;
                            case 0x22: s = "NO CIRCUIT/CHANNEL AVAILABLE";break;
                            case 0x26: s = "NETWORK OUT OF ORDER";       break;
                            case 0x29: s = "TEMPORARY FAILURE";          break;
                            case 0x2A: s = "SWITCH CONGESTION";          break;
                            case 0x2B: s = "ACCESS INFO DISCARD";        break;
                            case 0x2C: s = "REQ CIRCUIT/CHANNEL UNAVAIL";break;
                            case 0x2F: s = "RESOURCES UNAVAIL";          break;
                            case 0x31: s = "QUALITY OF SERVICE UNAVAIL"; break;
                            case 0x32: s = "REQ FACILITY NOT SUBSCRIBED";break;
                            case 0x36: s = "INCOMING CALL BARRED";       break;
                            case 0x39: s = "BEARER CAP NOT AUTHORIZED";  break;
                            case 0x3A: s = "BEARER CAP UNAVAILABLE";     break;
                            case 0x3F: s = "SERVICE/OPTION UNAVAILABLE"; break;
                            case 0x41: s = "BEARER CAP UNIMPLEMENTED";   break;
                            case 0x42: s = "CHANNEL TYPE UNIMPLEMENTED"; break;
                            case 0x45: s = "REQ FACILITY UNIMPLEMENTED"; break;
                            case 0x46: s = "ONLY RESTRICT AVAILABLE";    break;
                            case 0x4F: s = "SERVICE/OPTION UNIMPLEMENTED";break;
                            case 0x51: s = "INVALID CALL REF VALUE";     break;
                            case 0x53: s = "SUSPEND CALL, NO ID";        break;
                            case 0x54: s = "CALL ID IN USE";             break;
                            case 0x55: s = "NO CALL SUSPENDED";          break;
                            case 0x58: s = "INCOMPATIBLE DESTINATION";   break;
                            case 0x5B: s = "INVALID TRANSIT NET";        break;
                            case 0x5F: s = "INVALID MESSAGE";            break;
                            case 0x60: s = "MANDITORY IE MISSING";       break;
                            case 0x61: s = "MESSAGE TYPE UNIMPLEMENTED"; break;
                            case 0x62: s = "MESS INCOMP/T_ UNIMPLEMENTED";break;
                            case 0x63: s = "INFO ELEMENT UNIMPLEMENTED"; break;
                            case 0x64: s = "INVALID IE CONTENT";         break;
                            case 0x65: s = "MESSAGE INCOMPATIBLE";       break;
                            case 0x66: s = "RECOVERY ON TIMER EXPIRY";   break;
                            case 0x6F: s = "PROTOCOL ERROR";             break;
                            case 0x7F: s = "INTERNETWORKING";            break;
                            default:   s = "UNDEFINED";                  break;
                            }
                        sprintf(buf, "(%d) %s", *t & 0x7F, s);
#ifdef NEEDED
                        esinfo = buf + sprintf(buf, "%s", s);
                        if (!strcmp(s, "UNDEFINED"))
                            sprintf(esinfo, " [%02X]", *t & 0x7F);
#endif /* NEEDED */
                        esinfo = buf;
                        break;
                case 0x18:  /* Channel Identification */
                        msgstr = "CHANNEL ID";
                        if (data[2] & 0x08)
                            s = "EXCLUSIVE";
                        else
                            s = "PREFERRED";
                        strcpy(buf, s);
                        if (data[2] & 0x04)
                            strcat(buf, ", D CHANNEL");
                        else if ((data[2] & 0x20) == 0)  /* basic interface */
                            switch (data[2] & 0x03)
                                {
                                case 0x00: strcat(buf, ", NO B CHANNEL"); break;
                                case 0x01: strcat(buf, ", B1");           break;
                                case 0x02: strcat(buf, ", B2");           break;
                                case 0x03: strcat(buf, ", ANY");          break;
                                }
                        else
                            {
                            /* the decode for Primary Rate multi-channel */
                            /* slot maps, etc., goes here. */
                            BYTE *d = & data[3];
                            int err;
                            int j;
                            int chn;
                            enum {B, H0, H11, H12} slotmap;
                            /* skip the interface identifier, if any. */
                            if (data[2] & 0x40)
                                while ((*d++ & 0x80) == 0);
                            if ((*d & 0x60) != 0)  /* not CCITT encoding */
                                strcat(buf, ", (not CCITT)");
                            else 
                                {
                                err = OK;
                                switch (*d & 0x0F)
                                    {
                                    case 0x03: t=", B-";   slotmap = B;   break;
                                    case 0x06: t=", H0-";  slotmap = H0;  break;
                                    case 0x08: t=", H11-"; slotmap = H11; break;
                                    case 0x09: t=", H12-"; slotmap = H12; break;
                                    default: t = ""; err = ERR; break;
                                    }
                                strcat(buf, t);
                                t = buf + strlen(buf); 
                                if (err == OK && (*d++ & 0x10) == 0)
                                    {
                                    /* number is in following octet(s), so print
                                    ** them, comma separated.  If there are more
                                    ** than fit, put a "..." at the end.  Note
                                    ** that this does not check for passing the
                                    ** end of the IE... */
                                    while ((*d & 0x80) == 0 &&
                                             strlen(buf) + 5 <= EHM_MAX)
                                        t += sprintf(t, "%d, ", *d++);
                                    if ((*d & 0x80) == 0 ||
                                                strlen(buf) + 2 > EHM_MAX)
                                        strcat(buf, "...");
                                    else if (strlen(buf) + 2 <= EHM_MAX)
                                        sprintf(t, "%d", *d++ & 0x7F);
                                    }
                                if (err == OK && (*d++ & 0x10) == 1)
                                    {
                                    /* channel number is in slot map. */
                                    if (slotmap == B)
                                        {
                                        /* i = addr of low order entry */
                                        i = ie_len - (d - data) + 1;
                                        for (chn = 0; i >= 0; i--)
                                            for (j = 0; j < 8; j++, chn++)
                                                {
                                                if (d[i] & (1 << j))
                                                    t += sprintf(t,"%d, ", chn);
                                                if (strlen(buf) + 5 > EHM_MAX)
                                                    {
                                                    strcat(buf, "...");
                                                    break;
                                                    }
                                                }
                                        /* clean up end of string (axe trailing
                                        ** comma, truncate to EHM_MAX chars) */
                                        if (buf[strlen(buf) - 1] == '.')
                                            buf[EHM_MAX] = '\0';
                                        if (buf[strlen(buf) - 2] == ',')
                                            buf[strlen(buf) - 2] = '\0';
                                        else if (strlen(buf) < EHM_MAX)
                                            strcat(buf, "NONE"); /*no channels*/
                                        }
                                    if (slotmap == H0)
                                        {
                                        for (chn = 1, j = 0; j < 5; j++, chn++)
                                            {
                                            if (*d & (1 << j))
                                                t += sprintf(t, "%d, ", chn);
                                            }
                                        if (buf[strlen(buf) - 2] == ',')
                                            buf[strlen(buf) - 2] = '\0';
                                        else
                                            strcat(buf, "NONE");
                                        }
                                    if (slotmap == H11 || slotmap == H12)
                                        if (*d & 0x01)
                                            strcat(buf, "1");
                                        else
                                            strcat(buf, "NONE");

                                    }
                                }
                            }
                        buf[EHM_MAX] = '\0';
                        esinfo = buf;
                        break;
                case 0x29:  /* Date/time */
                        msgstr = "DATE/TIME";
                        sprintf(buf, "  %.2d:%.2d:%.2d:%.2d:%.2d:%.2d",
                                data[2], data[3], data[4],
                                data[5], data[6], data[7]);
                        esinfo = buf;
                        break;
                case 0x28:  /* Display */
                        msgstr = "DISPLAY";
                        sprintf(buf, "%.*s", MIN(ie_len, EHM_MAX), data + 2);
                        esinfo = buf;
                        break;
                case 0x7D:  /* High layer compatibility */
                        msgstr = "HIGH LAYER COMP";
                        break;
                case 0x2C:  /* Keypad facility */
                        msgstr = "KEYPAD FACILITY";
                        sprintf(buf, "%.*s", MIN(ie_len, EHM_MAX), data + 2);
                        esinfo = buf;
                        break;
                case 0x7C:  /* Low layer compatibility */
                        msgstr = "LOW LAYER COMP";
                        break;
                case 0x20:  /* Network-specific facilities */
                        msgstr = "NETWORK SPECIFIC";
                        break;
                case 0x27:  /* Notification Indicator */
                        msgstr = "NOTIFICATION IN";
                        switch (data[2] & 0x7F)
                            {
                            case 0x00: s = "USER SUSPENDED";       break;
                            case 0x01: s = "USER RESUMED";         break;
                            case 0x02: s = "BEARER SERVICE CHANGE";break;
                            default: s = "UNDEFINED [%02X]"; break;
                            }
                        sprintf(buf, s, data[2] & 0x7F);
                        esinfo = buf;
                        break;
                case 0x1E:  /* Progress Indicator */
                        msgstr = "PROGRESS INDICA";
                        switch (data[3] & 0x7F)
                            {
                            case 0x01: s = "NOT END-TO-END ISDN";  break;
                            case 0x02: s = "DESTINATION NON-ISDN"; break;
                            case 0x03: s = "ORIGINATION NON-ISDN"; break;
                            case 0x04: s = "CALL RETURNED TO ISDN";break;
                            case 0x05: s = "INTERNETWORKING SERVICE CHARGE";
                                    break;
                            case 0x08: s = "INFO/PATTERN NOW AVAILABLE";
                                    break; 
                            default: s = "UNDEFINED [%02X]"; break;
                            }
                        sprintf(buf, s, data[3] & 0x7F);
                        esinfo = buf;
                        break;
                case 0x79:  /* Restart indicator */
                        msgstr = "RESTART INDICAT";
                        break;
                case 0x00:  /* Segmented Message */
                        msgstr = "SEGMENTED MESSA";
                        break;
                case 0x34:  /* Signal */
                        msgstr = "SIGNAL";
                        switch (data[2])
                            {
                            case 0x00: s = "DIAL TONE ON";              break;
                            case 0x01: s = "RING BACK TONE ON";         break;
                            case 0x02: s = "INTERCEPT TONE ON";         break;
                            case 0x03: s = "NETWORK CONGESTION TONE ON"; break;
                            case 0x04: s = "BUSY TONE ON";              break;
                            case 0x05: s = "CONFIRM TONE ON";           break;
                            case 0x06: s = "ANSWER TONE ON";            break;
                            case 0x07: s = "CALL WAITING TONE";         break;
                            case 0x08: s = "OFF HOOK WARNING TONE";     break;
                            case 0x09: s = "PREEMPTION TONE ON";        break;
                            case 0x3F: s = "TONES OFF";                 break;
                            case 0x40: s = "ALERTING ON PATTERN 0";     break;
                            case 0x41: s = "ALERTING ON PATTERN 1";     break;
                            case 0x42: s = "ALERTING ON PATTERN 2";     break;
                            case 0x43: s = "ALERTING ON PATTERN 3";     break;
                            case 0x44: s = "ALERTING ON PATTERN 4";     break;
                            case 0x45: s = "ALERTING ON PATTERN 5";     break;
                            case 0x46: s = "ALERTING ON PATTERN 6";     break;
                            case 0x47: s = "ALERTING ON PATTERN 7";     break;
                            case 0x4F: s = "ALERTING OFF";              break;
                            default: s = "UNDEFINED [%02X]"; break;
                            }
                        sprintf(buf, s, data[2]);
                        esinfo = buf;
                        break;
                case 0x78:  /* Transit network selection */
                        msgstr = "TRANSIT NET SEL";
                        break;
                case 0x7E:  /* User-user */
                        msgstr = "USER-USER";
                        break;
                case 0x47:  /* Closed user group */
                        msgstr = "CLOSED USER GRO";
                        break;
                case 0x42:  /* End-to-end transit delay */
                        msgstr = "END-END DELAY";
                        break;
                case 0x40:  /* Information Rate */
                        msgstr = "INFORMATION RAT";
                        break;
                case 0x44:  /* Packet layer binary parameters */
                        msgstr = "BINARY PARAMETE";
                        break;
                case 0x45:  /* Packer layer window size */
                        msgstr = "WINDOW SIZE";
                        break;
                case 0x46:  /* Packet size */
                        msgstr = "PACKET SIZE";
                        break;
                case 0x74:  /* Redirecting number */
                        msgstr = "REDIRECTING NUM";
                        i = 0;
                        /* find out if the reason for redirection is in here */
                        if (data[2] & 0x80)
                            s = "UNKNOWN";  /* no extension byte with reason */
                        else if (data[3] & 0x80)
                            s = "UNKNOWN";  /* no extension byte with reason */
                        else switch (i = (data[4] & 0x0F))
                            {
                            case 0x00: s = "UNKNOWN";           break;
                            case 0x01: s = "BUSY";              break;
                            case 0x02: s = "NO REPLY";          break;
                            case 0x04: s = "DEFLECTION";        break;
                            case 0x09: s = "OUT OF ORDER";      break;
                            case 0x0A: s = "CALL FORWARDED";    break; 
                            case 0x0F: s = "REDIRECTED";        break; 
                            default: s = "UNDEFINED [%02X]"; break;
                            }
                        sprintf(buf, s, i);
                        esinfo = buf;
                        break;
                case 0x4A:  /* Reverse Charge indication */
                        msgstr = "REVERSE CHARGE";
                        break;
                case 0x43:  /* Transmit delay selection and indication */
                        msgstr = "DELAY SEL/IND";
                        break;
                case 0x7F:  /* Escape for extension */
                        msgstr = "ESCAPE EXTENTIO";
                        break;
                default:    /* Reserved... */
                        sprintf(buf, "UNDEFINED [%02X]", ie);
                        msgstr = buf;
                        break;
                }
            rtn = iprintf("        %04X %02X %02X %-15s",
                        offset, ie, ie_len, msgstr);
            if (rtn != OK)
                return rtn;
            if (ie_len == 0)
                rtn = iprintf(" : **ILLEGAL ZERO-LENGTH INFORMATION ELEMENT\n");
            else if (strlen(esinfo))
                rtn = iprintf(" : %s\n", esinfo);
            else
                rtn = iprintf("\n");
            if (rtn != OK)
                return rtn;
            data += ie_len + 2;
            offset += ie_len + 2;
            }
        else    /* multiple byte Information Element from unknown codeset */
            {
            /* codeset is not 0: put out an "I don't know" msg */
            ie_len = data[1];
            rtn = iprintf("        %04X %02X %02X                "
                          " : UNKNOWN CODESET %d\n",
                         offset, ie, ie_len, shifted_codeset);
            if (rtn != OK)
                return rtn;
            shifted_codeset = active_codeset;
            data += ie_len + 2;
            offset += ie_len + 2;
            }

        }

    return OK;
}


int hex_dump(char *data, int datalen, char *outbuf, long outbuflen)

{
    int rtn;
    
    if (HEXDUMPLEN(datalen) > outbuflen)
        return ERR_NO_MEM;

    id_outbuf = outbuf;
    id_outbuf_sz = outbuflen;
    
    rtn = hdump((BYTE *)data, datalen);

    return rtn;
}



static int hdump(BYTE *data, int datalen)

{
    int offset = 0;
    int i;
    int rtn = OK;
#define H(x) data[(x)]    
#define C(x) (isprint(data[(x)]) ? data[(x)] : '.')

    if (HEXDUMPLEN(datalen) > id_outbuf_sz)
        return ERR_NO_MEM;
    for ( ; datalen >= 16 ; data += 16, datalen -= 16, offset += 16)
        rtn = iprintf("    %04X: %02X %02X %02X %02X  "
                "%02X %02X %02X %02X  %02X %02X %02X %02X  %02X %02X %02X %02X"
                "  %c%c%c%c%c%c%c%c%c%c%c%c%c%c%c%c\n", offset,
                H(0), H(1), H(2), H(3),   H(4), H(5), H(6), H(7),
                H(8), H(9), H(10), H(11), H(12), H(13), H(14), H(15),
                C(0), C(1), C(2), C(3),   C(4), C(5), C(6), C(7),
                C(8), C(9), C(10), C(11), C(12), C(13), C(14), C(15) );
    if (rtn != OK)
        return rtn;
    if (datalen > 0)
        {
        rtn = iprintf("    %04X: ", offset);
    if (rtn != OK)
        return rtn;
        for (i = 0; rtn == OK && i < 16; i++)
            {
            if (i < datalen)
                iprintf("%02X ", H(i));
            else
                {
                *id_outbuf++ = ' ';
                *id_outbuf++ = ' ';
                *id_outbuf++ = ' ';
                id_outbuf_sz -= 3;
                }
            if ((i & 3) == 3)
                {
                *id_outbuf++ = ' ';
                id_outbuf_sz--;
                }
            }
        for (i = 0; i < datalen; i++)
            {
            *id_outbuf++ = C(i);
            id_outbuf_sz--;
            }
        *id_outbuf++ = '\n';
        id_outbuf_sz--;
        }
    *id_outbuf = '\0';
    return OK;
}


#ifdef MAKE_TEST_PROGRAM

#define IDP_MAX_RX_LEN 280

enum ErrCodes { ERROR_SYNTAX=0x0800,
                ERROR_NO_DATA,
                ERROR_NO_MEM,
                ERROR_CORRUPT_FILE };

#define SYNTAX_ERR  "Syntax Error in command line\n"
#define USAGE       "USAGE: isdnshow [infilename] [-o outfilename]\n"

static int Debug=0;
static int ascii_only=0;

/* isdnshow [infilename] [-o outfilename]  */
int main ( int argc, char *argv[])
{
char        *inFileName, *outFileName, *obuf;
int         i, padding = 0;
FILE        *outfp, *infp;
unsigned int *check;
short       obuf_size = 5000;  /* arbitrary buffer size */

TLF_DCH_RECORD      *TrecP;
TLF_LOG_FILE_HEADER hbuf;

    outfp = stdout;
    infp = stdin;
    inFileName = outFileName = NULL;
    
#ifdef NOT
    fprintf(stderr, "\nIsdnshow, version 1.0.2\n");
    fprintf(stderr, "Copyright (C) 1996, by Digi International, All rights reserved.\n");
#endif

    for (i=1,--argc; argc && strlen(argv[i]); --argc, i++)
        {
        if ( *argv[i] == '-' && toupper (*(argv[i]+1)) == 'O')
            {
            i++;
            if ( strlen (argv[i]))
                outFileName = argv[i];
            else
                return fprintf (stderr, "%s\n%s", SYNTAX_ERR, USAGE);
            continue;
            }
        if ( *argv[i] == '-' && toupper (*(argv[i]+1)) == 'D')
            {
            Debug = YES;
            continue;
            }
        if ( *argv[i] == '-' && toupper (*(argv[i]+1)) == 'A')
            {
            ascii_only = YES;
            continue;
            }
        if ( *argv[i] == '-' )
            {
            fprintf (stderr, "%s\n%s\n", SYNTAX_ERR, USAGE);
            return ERROR_SYNTAX;
            }
        inFileName = argv[i];
        }

    if (    ((obuf = (char *)malloc(obuf_size)) == NULL )
        ||  ((TrecP = (TLF_DCH_RECORD *)malloc
                        ( sizeof(TLF_DCH_RECORD) + IDP_MAX_RX_LEN))== NULL))

        {
        fprintf (stderr, "failed to malloc!\n");
        return ERROR_NO_MEM;
        }

    if ( inFileName )
        {
        if ( (infp = fopen (inFileName, "rb")) == NULL )
            return fprintf (stderr, "Failed to open <%s> for input\n%s", 
                    inFileName, USAGE );
        }
#ifdef NOT
    else
        fprintf(stderr, "Reading from standard input...\n");
#endif

    if ( outFileName )
        if ( (outfp = fopen (outFileName, "wb")) == NULL )
            return fprintf (stderr, "Failed to open <%s> for output\n%s", 
                    outFileName, USAGE );

    /* Beginning of file MUST be our magic LogFileHeader record */
    if (    ( fread ( &hbuf, sizeof(TLF_LOG_FILE_HEADER), 1, infp ) != 1 )
        || (strncmp ( hbuf.Title, TLF_LOG_TITLE, TLF_LOG_TITLE_CHECK_SIZE)))
    {
        fprintf (stderr, "Failed to read header record from input file\n");
        if ( inFileName )
            fclose ( infp );
        if ( outFileName )
            fclose ( outfp );
        return ERROR_SYNTAX;
    }
    /* print title record, skipping last character (should be ctrl-Z) */
    fprintf(outfp, "%.*s\n", (int)strlen(hbuf.Title) - 1, hbuf.Title);
    

    /* begin looping: read input record, format to output */
    while (1)
        {
        /* read the record type independent parts of the trace record */
        if ( fread ( TrecP, sizeof(TLF_HEADER), 1, infp ) != 1 )
            break;
        /* check for a known record type */
        switch (TrecP->Header.TypeId)
            {
            case TLF_TYPE_DCH:
            case TLF_TYPE_BCH:
            case TLF_TYPE_HEX:
            case TLF_TYPE_PPP:
            case TLF_TYPE_X25:
            case TLF_TYPE_ASCII:
                        break;
            default:
                        fprintf (stderr,"Unknown Data Record Type %d!\n",
                                 TrecP->Header.TypeId);
                        fprintf (stderr,"addr: %x %x %x %x %x %x %x %x\n",
                                 ((unsigned char *)TrecP + 0),
                                 ((unsigned char *)TrecP + 1),
                                 ((unsigned char *)TrecP + 2),
                                 ((unsigned char *)TrecP + 3),
                                 ((unsigned char *)TrecP + 4),
                                 ((unsigned char *)TrecP + 5),
                                 ((unsigned char *)TrecP + 6),
                                 ((unsigned char *)TrecP + 7));
                        fprintf (stderr,"dump: %x %x %x %x %x %x %x %x\n",
                                 *((unsigned char *)TrecP + 0),
                                 *((unsigned char *)TrecP + 1),
                                 *((unsigned char *)TrecP + 2),
                                 *((unsigned char *)TrecP + 3),
                                 *((unsigned char *)TrecP + 4),
                                 *((unsigned char *)TrecP + 5),
                                 *((unsigned char *)TrecP + 6),
                                 *((unsigned char *)TrecP + 7));
                        goto err_exit;
            }
        /* is trace record shorter than the header or longer than buffer */
        if ( NOSWAP32(TrecP->Header.Len) > obuf_size ||
             NOSWAP32(TrecP->Header.Len) < sizeof(TLF_HEADER)+sizeof (TLF_LENGTH))
            {
            fprintf (stderr,"Unuseable Data Record length %d!\n",
                NOSWAP32(TrecP->Header.Len));
            goto err_exit;
            }
        /* read the remainder of the trace record */
	if ( fread ( ((char *)TrecP) + sizeof(TLF_HEADER),
		NOSWAP32(TrecP->Header.Len) - sizeof(TLF_HEADER), 
		1, infp ) != 1 )
	{
		fprintf (stderr,"Could not read data portion of record!\n");
		goto err_exit;
	}

	check =	(unsigned int *) (((char *) TrecP) +
		NOSWAP32(TrecP->Header.Len) - sizeof(TLF_LENGTH)) ;
	/* Add padding for proper alignment */
	if (TrecP->DataLen%sizeof(int)) {
		padding = sizeof(int) - (TrecP->DataLen%sizeof(int));
	}
	else
		padding = 0;

        /* consistency check: length fields at start and end must match */
        if (NOSWAP32(TrecP->Header.Len) != NOSWAP32(*check))
	{
		fprintf (stderr,"Inconsistent record contents!\n");
#		ifdef DEBUG
		{
		TLF_HEADER *te = (TLF_HEADER *)TrecP;
		fprintf(outfp,
		"Len=%lu TypeId=%u SeqNum=%d TimeStamp=%.2d:%.2d:%.2d.%.2d\n",
			NOSWAP32(te->Len), NOSWAP16(te->TypeId), 
			NOSWAP16(te->SeqNum), te->TimeStamp.Hour, 
			te->TimeStamp.Minute, te->TimeStamp.Second,
			te->TimeStamp.CentiSecond);

		fprintf(outfp, "DeviceId=%d LineId=%d\n",
			te->DeviceId, te->LineId);
		fprintf(outfp, "dch record size=%d\n",sizeof(TLF_DCH_RECORD));
		fprintf(outfp, "TrecP=%08lX offset=%08lX End len ptr=%08lx\n", 
			TrecP, NOSWAP32(TrecP->Header.Len)-sizeof(TLF_LENGTH),
			NOSWAP32((((char *) TrecP) 
			[TrecP->Header.Len - sizeof(TLF_LENGTH)])));
		hex_dump((char *)te, NOSWAP32(te->Len), obuf, obuf_size);
		fputs(obuf, outfp);
		fflush(outfp);
		}
#		endif
		goto err_exit;
	}
	if (feof (infp))
		break;

	/* do record type specific integrity checks... */
        switch ( NOSWAP16(TrecP->Header.TypeId ))
            {
            case TLF_TYPE_DCH:
                if (Debug)
                    fprintf (stderr,"DCH record #%d\n",
                              NOSWAP16(TrecP->Header.SeqNum));
/* some consistency checks:ThisLength MUST=sizeof (TLF_DCH_RECORD)+datalen, */
/* and datalen MUST <= IDP_MAX_RX_LEN */
                if (    (NOSWAP32(TrecP->Header.Len) != (sizeof (TLF_DCH_RECORD)
                             + NOSWAP32(TrecP->DataLen) + sizeof(TLF_LENGTH) 
				+ padding))
                        ||  ( NOSWAP32(TrecP->DataLen) > IDP_MAX_RX_LEN ) )
                    {
                    fprintf (stderr,"Inconsistent D-Channel Record!\n");
fprintf(stderr,"%x != %x %x %x %x\n",
                TrecP->Header.Len, sizeof (TLF_DCH_RECORD),
                             TrecP->DataLen, sizeof(TLF_LENGTH), padding);
                    goto err_exit;
                    }
                break;
            case TLF_TYPE_ASCII:
                if (    NOSWAP32(TrecP->Header.Len) != 
                        (sizeof (TLF_ASCII_RECORD) + sizeof(TLF_LENGTH) + 
                        NOSWAP32(((TLF_ASCII_RECORD *)TrecP)->DataLen)))
                    {
                    fprintf (stderr,"Inconsistent ASCII Record!\n");
                    goto err_exit;
                    }
                break;
            case TLF_TYPE_HEX:
                if (    SWAP32(TrecP->Header.Len) != 
                        (sizeof (TLF_ASCII_RECORD) + sizeof(TLF_LENGTH) + 
                        SWAP32(((TLF_HEX_RECORD *)TrecP)->DataLen)))
                    {
                    fprintf (stderr,"Inconsistent Hex Record!\n");
                    goto err_exit;
                    }
                break;

            /* these record types not defined yet, formatter will hex dump... */
            case TLF_TYPE_BCH:
            case TLF_TYPE_PPP:
            case TLF_TYPE_X25:
            default:
                break;
            }

		/* if ascii_only, then just print ASCII messages (for debugging) */
		if (ascii_only && (NOSWAP16(TrecP->Header.TypeId) != TLF_TYPE_ASCII))  
				continue;

        /* Send if off to formatter */
        if ( isdn_dump((TLF_HEADER *)TrecP, obuf, obuf_size,
                IDF_MSG_HEADER+IDF_INFO_HEADER+IDF_HEX_DUMP+
                +IDF_WHITESPACE+IDF_INFO_ELEMENTS+IDF_NO_ASCII_HEX_DUMP) )
            {
            fprintf(stderr,"Failed to format raw trace event record\n");
            break;
            }

        /* Send the formatted buffer to destination */
        if (fputs ( obuf, outfp ) == EOF )
            {
            fprintf(stderr,"Failed to output formatted trace event!\n");
            break;
            }
        }

err_exit:
    if ( inFileName )
        fclose ( infp );
    if ( outFileName )
        fclose ( outfp );
    return 0;
}

#endif
